WITH RR_Type;
WITH Dns_Types;
WITH Dns_Network;
WITH Unsigned_Types;
with ada.real_time;
--# inherit rr_type, dns_types, dns_network, unsigned_types, ada.real_time;
PACKAGE Buffer_Pkg IS
   TYPE Occupied_Status_Type IS (FREE,OCCUPIED);
   TYPE BufferEntryType IS RECORD
      Msg: Dns_Types.DNS_Message; --DNS query or response
      NumBytes : Dns_Types.Message_Length_Range; --length of msg in bytes
      Requester:  Dns_Network.Network_Address_And_Port; --Who's asking?

      --for external queries, the ID and port number of the
      --corresponding query we send upstream
      ConnectingID : Dns_Types.Unsigned_Short;
      PortNumber:  integer;

      --for external queries, the original and randomized case versions
      -- of the domain name in the corresponding query we send upstream
      OriginalCaseDomainName : RR_Type.WireStringType;
      RandomizedCaseDomainName : RR_Type.WireStringType;

      Status : Occupied_Status_Type;
      TimeStamp : Ada.Real_Time.Time; --used to expire entries
   END RECORD;
   EMPTY_BUFFER_ENTRY : CONSTANT BufferEntryType :=
      BufferEntryType'(Msg => Dns_Types.EMPTY_MSG,
      NumBytes => 0,
      Requester => Dns_Network.NO_NETWORK_ADDRESS_AND_PORT,
      ConnectingID => 0,
      PortNumber => 0,
      OriginalCaseDomainName => RR_Type.BlankWire,
      RandomizedCaseDomainName => RR_Type.BlankWire,
      Status => FREE,
      TimeStamp => Ada.Real_Time.Time_First);

   --change this number to increase the query and response buffer sizes
   BufferSize : CONSTANT unsigned_types.Unsigned32 := 4096;

   SUBTYPE BufferIndexWithZeroType IS unsigned_types.Unsigned32 RANGE 0..BufferSize;
   SUBTYPE BufferIndexType IS unsigned_types.Unsigned32 RANGE 1..BufferSize;
   type BufferType is array(BufferIndexType) of BufferEntryType;
END Buffer_Pkg;

