WITH Spark_Regpat;
WITH Ada.Text_IO;
PACKAGE BODY Check_Patterns IS
   --# hide Check_Patterns;
   MAX_PATTERNS : CONSTANT Natural := 1024;
   SUBTYPE Pattern_Range IS Natural RANGE 1..MAX_PATTERNS;
   SUBTYPE Line_Index IS Natural RANGE 1..256;
   subtype Line_Type is String(Line_Index);
   type Pattern_Array is array(Pattern_Range) of Spark_Regpat.Gnat_Pattern;

   Good_Patterns : Pattern_Array;
   Number_Good_Patterns : Natural := 0;
   Bad_Patterns : Pattern_Array;
   Number_Bad_Patterns : Natural := 0;


   --spark_regpat.compile(pattern,"[0-9]+\.");
   --spark_regpat.compile(pattern2,"^([0-9]|[a-f]|[A-F])+\.[0-9]+\.");
   --spark_regpat.compile(pattern3,"in-addr\.arpa");

   PROCEDURE Process_File(Filename : IN String; Patterns: IN OUT Pattern_Array;
         Number_Patterns : IN OUT Natural) IS
      File : Ada.Text_IO.File_Type;
      CurrentLine : Line_type;
      LastPos : Natural;
   BEGIN
      Ada.Text_IO.Open(File => File, Mode => Ada.Text_IO.In_File,
         Name => Filename);
      WHILE not Ada.Text_IO.End_Of_File(file) LOOP
         Ada.Text_IO.Get_Line(File => File,
            Item => CurrentLine, Last => LastPos);
         Number_Patterns := Number_Patterns + 1;
         spark_regpat.compile(Patterns(Number_Patterns),currentline(1..lastpos));
      END LOOP;
      Ada.Text_IO.Close(file);
   end Process_File;

   -----------------------------
   -- Initialize_Bad_Patterns --
   -----------------------------

   procedure Initialize_Bad_Patterns
      (Blacklist : IN String)   IS
   BEGIN
      Process_File(Blacklist,Bad_Patterns,Number_Bad_Patterns);
   end Initialize_Bad_Patterns;

   -----------------------------
   -- Initialize_Good_Patterns --
   -----------------------------

   procedure Initialize_Good_Patterns
     (Whitelist : IN String)
   is

   begin
      Process_File(Whitelist,Good_Patterns,Number_Good_Patterns);
   end Initialize_Good_Patterns;


   ----------------------
   -- Suspicious_Query --
   ----------------------

   FUNCTION Suspicious_Query (Domainname : IN String) RETURN Boolean IS
      Found : boolean := False;
   BEGIN
      for i in Natural range 1..Number_Good_Patterns loop
         IF Spark_Regpat.Match(Good_Patterns(I),Domainname) THEN
            RETURN False;
         END IF;
      END LOOP;

      for i in Natural range 1..Number_Bad_Patterns loop
         IF Spark_Regpat.Match(Bad_Patterns(I),Domainname) THEN
            RETURN True;
         END IF;
      end loop;
      RETURN False;
   EXCEPTION WHEN OTHERS =>
      return false;
   end Suspicious_Query;

end Check_Patterns;
