----------------------------------------------------------------
-- IRONSIDES - DNS SERVER
--
-- By: Martin C. Carlisle and Barry S. Fagin
--     Department of Computer Science
--     United States Air Force Academy
--
-- This is free software; you can redistribute it and/or
-- modify without restriction.  We do ask that you please keep
-- the original author information, and clearly indicate if the
-- software has been modified.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
----------------------------------------------------------------
--WITH ada.Text_IO;
WITH Dns_Types;
use type dns_types.message_length_range;

package body Dns_Network_Receive is

   -------------------------
   -- Receive_DNS_Message --
   -------------------------

   PROCEDURE Receive_DNS_Message(
      Message : out DNS_Types.DNS_Message;
      Number_Bytes  : out DNS_Types.message_Length_Range;
      Reply_Address : OUT DNS_Network.Network_Address_And_Port;
      Arriving_Port : OUT Integer;
      Failure : OUT Boolean;
      Timeout : OUT Boolean)
   IS
   BEGIN
      DNS_Network.Receive_DNS_Message(
         Message       => Message,
         Number_Bytes  => Number_Bytes,
         Reply_Address => Reply_Address,
         Arriving_Port => Arriving_Port,
         Failure       => Failure,
         Timeout       => Timeout);
      if not Failure then
         Failure := (Number_Bytes < DNS_Types.message_Length_Range(1+DNS_Types.Header_Bits/8));
            --OR (Number_Bytes > DNS_Network.MAX_QUERY_SIZE);
            --Above is wrong if DNS message is DNSSEC-sized.  What does SPARK say?
         --Ada.Text_IO.put("Number_Bytes = " & dns_types.message_length_range'image(Number_Bytes));
      END IF;
   end Receive_DNS_Message;


end Dns_Network_Receive;
