----------------------------------------------------------------
-- IRONSIDES - DNS SERVER
--
-- By: Martin C. Carlisle and Barry S. Fagin
--     Department of Computer Science
--     United States Air Force Academy
--
-- This is free software; you can redistribute it and/or
-- modify without restriction.  We do ask that you please keep
-- the original author information, and clearly indicate if the
-- software has been modified.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
----------------------------------------------------------------
--with ada.text_io;
WITH Ada.Real_Time, Ada.Characters.Handling, Ada.Unchecked_Conversion,
   Unsigned_Types, Dns_Types, rr_type, rr_type.a_record_type, rr_type.aaaa_record_type,
   Rr_Type.Cname_Record_Type, Rr_Type.Dnskey_Record_Type, Rr_Type.Mx_Record_Type,
   Rr_Type.Ns_Record_Type, Rr_Type.NXDOMAIN_Record_Type, Rr_Type.Nsec_Record_Type,
   Rr_Type.Ptr_Record_Type, Rr_Type.rrsig_record_type, Rr_Type.Soa_Record_Type,
     Rr_Type.Corrupt_Record_Type, Protected_SPARK_IO_05, Ada.Text_IO;


USE TYPE Unsigned_Types.Unsigned16;
USE TYPE Unsigned_Types.Unsigned32;
USE TYPE Ada.Real_Time.Time;
USE TYPE Ada.Real_Time.Time_Span;
USE TYPE Dns_Types.Message_Bytes_Range;
USE TYPE Dns_Types.Message_Length_Range;
USE TYPE Dns_Types.Byte;
use type dns_types.Query_Class;

--need these for equality operator in package body
use type Rr_Type.a_Record_Type.ARecordType;
use type rr_type.aaaa_record_type.AAAARecordType;
USE TYPE Rr_Type.Cname_Record_Type.CNAMERecordType;
USE TYPE Rr_Type.Dnskey_Record_Type.DNSKEYRecordType;
USE TYPE Rr_Type.Mx_Record_Type.MXRecordType;
use type Rr_Type.Ns_Record_Type.NSRecordType;
USE TYPE Rr_Type.NSEC_Record_Type.NSECRecordType;
USE TYPE Rr_Type.NXDOMAIN_Record_Type.NXDOMAINRecordType;
use type Rr_Type.Ptr_Record_Type.PTRRecordType;
use type Rr_Type.rrsig_record_type.RRSIGRecordType;
USE TYPE Rr_Type.Soa_Record_Type.SOARecordType;
USE TYPE Rr_Type.Corrupt_Record_Type.CorruptRecordType;

--# inherit Ada.Real_Time, System, unsigned_types, Ada.Characters.Handling,
--#    Ada.Unchecked_Conversion, dns_types, rr_type, rr_type.a_record_type, rr_type.aaaa_record_type,
--#    rr_type.cname_record_type, rr_type.dnskey_record_type, rr_type.mx_record_type,
--#    rr_type.ns_record_type, rr_type.nsec_record_type, rr_type.nxdomain_record_type, rr_type.ptr_record_type,
--#    rr_type.rrsig_record_type, rr_type.Corrupt_Record_Type, rr_type.soa_record_type;

package dns_table_pkg
--# own protected DNS_Table : DNS_Table_Type (priority => 0);
IS

protected type Dns_Table_Type is

   pragma priority(0);

--insertion methods
   procedure insertARecord(Key : in Rr_Type.WireStringType;
                           theRecord : in rr_type.a_record_type.ARecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;




   procedure InsertCorruptRecord(Key : in Rr_Type.WireStringType;
                          theRecord : in rr_type.corrupt_record_type.CorruptRecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;

   procedure insertAAAARecord(Key : in Rr_Type.WireStringType;
                           theRecord : in rr_type.aaaa_record_type.AAAARecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;

   procedure insertCNAMERecord(Key : in Rr_Type.WireStringType;
                           theRecord : in rr_type.cname_record_type.CNAMERecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;

   procedure InsertDNSKEYRecord(Key : in Rr_Type.WireStringType;
                          theRecord : in rr_type.DNSKEY_record_type.DNSKEYRecordType);
   --# global in Ada.Real_Time.ClockTime;
   --# in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;

   procedure insertMXRecord(Key : in Rr_Type.WireStringType;
                            theRecord : in rr_type.mx_record_type.MXRecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;

   procedure insertNSRecord(Key : in Rr_Type.WireStringType;
                           theRecord : in rr_type.ns_record_type.NSRecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;

   procedure insertNSECRecord(Key : in Rr_Type.WireStringType;
                           theRecord : in rr_type.nsec_record_type.NSECRecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;
   --# pre theRecord.numberOfBlocks > 0;

   procedure insertNXDOMAINRecord(Key : in Rr_Type.WireStringType;
                           theRecord : in rr_type.nxdomain_record_type.NXDOMAINRecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;

   procedure insertPTRRecord(Key : in Rr_Type.WireStringType;
                           theRecord : in rr_type.ptr_record_type.PTRRecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;

   procedure InsertRRSIGRecord(Key : in Rr_Type.WireStringType;
                          theRecord : in rr_type.RRSIG_record_type.RRSIGRecordType);
   --# global in Ada.Real_Time.ClockTime;
   --#    in out DNS_Table_Type;
   --# derives DNS_Table_Type from *, Ada.Real_Time.ClockTime, theRecord, Key;

   procedure InsertSOARecord(Key : in Rr_Type.WireStringType;
                           theRecord : in rr_type.soa_record_type.SOARecordType);
   --# global in out DNS_Table_Type; in Ada.Real_Time.ClockTime;
   --# derives DNS_Table_Type from *, theRecord, Key, Ada.Real_Time.ClockTime;

   procedure queryARecords(
   	domainName : in rr_type.WireStringType;
        returnedRecords : out rr_type.a_record_type.ARecordBucketType;
        howMany : out rr_type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;





   procedure queryCorruptRecords(
   	domainName : in rr_type.WireStringType;
        inTable : out Boolean);
   --# global in DNS_Table_Type;
   --# derives inTable from DNS_Table_Type, domainName;




   procedure queryAAAARecords(
   	domainName : in rr_type.WireStringType;
        returnedRecords : out rr_type.aaaa_record_type.AAAARecordBucketType;
        howMany : out rr_type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   procedure queryCNAMERecords(
      domainName : in rr_type.WireStringType;
      returnedRecords : out rr_type.cname_record_type.CNAMERecordBucketType;
      howMany : out rr_type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   procedure queryDNSKEYRecords(
      domainName : in rr_type.WireStringType;
      returnedRecords : out rr_type.DNSKEY_record_type.DNSKEYRecordBucketType;
      HowMany : out Rr_Type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   procedure queryMXRecords(
      domainName : in rr_type.WireStringType;
      returnedRecords : out rr_type.mx_record_type.mxRecordBucketType;
      howMany : out rr_type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   procedure queryNSRecords(
      domainName : in rr_type.WireStringType;
      returnedRecords : out rr_type.ns_record_type.NSRecordBucketType;
      howMany : out rr_type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   procedure queryNSECRecords(
      domainName : in rr_type.WireStringType;
      returnedRecords : out rr_type.nsec_record_type.NSECRecordBucketType;
      howMany : out rr_type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   procedure queryNXDOMAINRecords(
      domainName : in rr_type.WireStringType;
      returnedRecords : out rr_type.nxdomain_record_type.NXDOMAINRecordBucketType;
      howMany : out rr_type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   procedure queryPTRRecords(
      domainName : in rr_type.WireStringType;
      returnedRecords : out rr_type.ptr_record_type.PTRRecordBucketType;
      howMany : out rr_type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   procedure queryRRSIGRecords(
      domainName : in rr_type.WireStringType;
      returnedRecords : out rr_type.RRSIG_record_type.RRSIGRecordBucketType;
      HowMany : out Rr_Type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   procedure querySOARecords(
      domainName : in rr_type.WireStringType;
      returnedRecords : out rr_type.soa_record_type.SOARecordBucketType;
      howMany : out rr_type.NumberOfRecordsType);
   --# global in DNS_Table_Type, Ada.Real_Time.ClockTime;
   --# derives returnedRecords from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime &
   --#         howMany from DNS_Table_Type, domainName, Ada.Real_Time.ClockTime;

   private
      ARecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      ARecordTable : Rr_Type.A_Record_Type.ARecordHashTableType := rr_type.a_record_type.ARecordHashTableType'(
         others => rr_type.a_record_type.ARecordBucketType'(
          others => rr_type.a_record_type.blankARecord));



      CorruptRecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      CorruptRecordTable : Rr_Type.Corrupt_Record_Type.CorruptRecordHashTableType := rr_type.Corrupt_Record_Type.CorruptRecordHashTableType'(
         others => rr_type.Corrupt_Record_Type.CorruptRecordBucketType'(
         others => rr_type.Corrupt_Record_Type.BlankRecord));


      AAAARecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      AAAARecordTable : Rr_Type.AAAA_Record_Type.AAAARecordHashTableType := rr_type.aaaa_record_type.AAAARecordHashTableType'(
         others => rr_type.aaaa_record_type.AAAARecordBucketType'(
         others => rr_type.aaaa_record_type.blankAAAARecord));
      CNAMERecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      CNAMERecordTable : Rr_Type.Cname_Record_Type.CNAMERecordHashTableType := rr_type.cname_record_type.CNAMERecordHashTableType'(
         others => rr_type.cname_record_type.CNAMERecordBucketType'(
         others => Rr_Type.Cname_Record_Type.BlankCNAMERecord));
      DNSKEYRecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      DNSKEYRecordTable : Rr_Type.dnskey_Record_Type.DNSKEYRecordHashTableType := rr_type.dnskey_record_type.DNSKEYRecordHashTableType'(
         others => rr_type.dnskey_record_type.DNSKEYRecordBucketType'(
         others => rr_type.dnskey_record_type.blankDNSKEYRecord));
      MXRecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
         others => RR_Type.BlankOwner));
      MXRecordTable : Rr_Type.Mx_Record_Type.MXRecordHashTableType := rr_type.mx_record_type.MXRecordHashTableType'(
         others => rr_type.mx_record_type.MXRecordBucketType'(
         others => rr_type.mx_record_type.blankMXRecord));
      NSRecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      NSRecordTable : rr_type.ns_record_type.NSRecordHashTableType := rr_type.ns_record_type.NSRecordHashTableType'(
         others => rr_type.ns_record_type.NSRecordBucketType'(
            others => Rr_Type.Ns_Record_Type.BlankNSRecord));
      NSECRecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      NSECRecordTable : rr_type.nsec_record_type.NSECRecordHashTableType := rr_type.nsec_record_type.NSECRecordHashTableType'(
         others => rr_type.nsec_record_type.NSECRecordBucketType'(
            OTHERS => Rr_Type.Nsec_Record_Type.BlankNSECRecord));
      NXDOMAINRecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      NXDOMAINRecordTable : Rr_Type.Nxdomain_Record_Type.NXDOMAINRecordHashTableType := rr_type.nxdomain_record_type.NXDOMAINRecordHashTableType'(
         others => rr_type.nxdomain_record_type.NXDOMAINRecordBucketType'(
            others => Rr_Type.Nxdomain_Record_Type.BlankNXDOMAINRecord));
      PTRRecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      PTRRecordTable : rr_type.ptr_record_type.PTRRecordHashTableType := rr_type.ptr_record_type.PTRRecordHashTableType'(
         others => rr_type.ptr_record_type.PTRRecordBucketType'(
            others => Rr_Type.Ptr_Record_Type.BlankPTRRecord));
      RRSIGRecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      RRSIGRecordTable : RR_Type.rrsig_Record_Type.rrsigRecordHashTableType := rr_type.rrsig_record_type.rrsigRecordHashTableType'(
         others => rr_type.rrsig_record_type.rrsigRecordBucketType'(
            others => rr_type.rrsig_record_type.blankrrsigRecord));
      SOARecordKeys : RR_Type.OwnerHashTableType := RR_Type.OwnerHashTableType'(
         others => RR_Type.OwnerRecordBucketType'(
            others => RR_Type.BlankOwner));
      SOARecordTable : rr_type.SOA_record_type.SOARecordHashTableType := rr_type.SOA_record_type.SOARecordHashTableType'(
         others => rr_type.SOA_record_type.SOARecordBucketType'(
          others => rr_type.SOA_record_type.blankSOARecord));
   end Dns_Table_Type;

--THIS IS THE NAME SERVER HASH TABLE
DNS_Table: Dns_Table_Type;

PRIVATE
FUNCTION Seconds_To_Timespan(Seconds: Integer) RETURN Ada.Real_Time.Time_Span;

end dns_table_pkg;
