----------------------------------------------------------------
-- IRONSIDES - DNS SERVER
--
-- By: Martin C. Carlisle and Barry S. Fagin
--     Department of Computer Science
--     United States Air Force Academy
--
-- This is free software; you can redistribute it and/or
-- modify without restriction.  We do ask that you please keep
-- the original author information, and clearly indicate if the
-- software has been modified.
--
-- This software is distributed in the hope that it will be useful,
-- but WITHOUT ANY WARRANTY; without even the implied warranty
-- of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
----------------------------------------------------------------

with Gnat.Byte_Swapping;
package body DNS_Types is

   function Byte_Swap_US(U : Unsigned_Short) return Unsigned_Short is
      --# hide Byte_Swap_US
      Answer : Unsigned_Short;
   begin
      --
      Answer := U;
      Gnat.Byte_Swapping.Swap2(Answer'Address);
      return Answer;
   end Byte_Swap_US;

   ---------------
   -- Byte_Swap --
   ---------------

   procedure Byte_Swap (H : in out Header_Type) is
   begin
      H.MessageID := Byte_Swap_US(H.MessageID);
      H.QDCount := Byte_Swap_US(H.QDCount);
      H.ANCount := Byte_Swap_US(H.ANCount);
      H.NSCount := Byte_Swap_US(H.NSCount);
      H.ARCount := Byte_Swap_US(H.ARCount);
   end Byte_Swap;

   FUNCTION Is_Query(H: IN Header_Type) RETURN Boolean IS
   BEGIN
      return H.QR = false;  --if this bit is zero, msg is query
   END Is_Query;

   PROCEDURE Print_Header_Info(H : IN Header_Type)
   IS
   BEGIN
      Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
         Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
         "MessageID = ",0);
      Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
         Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
         Integer(H.MessageID),2,10);
      Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
         Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
         " QR = ",0);
      CASE H.QR IS
         when false =>
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               0,2,10);
         WHEN True =>
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               1,2,10);
      end case;
      --can add more as needed
   END Print_Header_Info;

end DNS_Types;
