WITH Ada.Text_IO;
WITH Gnat.Os_Lib;
WITH Ada.Command_Line;
with check_patterns;
package body Non_Spark_Command_Line_Pkg is
--# hide Non_Spark_Command_Line_Pkg; --SPARK doesn't understand ada.command_line
   -------------------------------
   -- Process_Command_Line_Args --
   -------------------------------

   Google_DNS_IP : aliased String := "8.8.8.8";
   Level3_DNS_IP : aliased String := "209.244.0.3";
   USAFA_DNS_IP : aliased String := "128.236.1.45";
   BSF_DNS_IP : aliased String := "75.75.75.75";
   IP_TO_BE_USED : MyString := Level3_DNS_IP'access;

   FUNCTION IP_TO_BE_USED_STRING RETURN MyString IS
   BEGIN
      return IP_TO_BE_USED;
   END IP_TO_BE_USED_STRING;

   PROCEDURE Usage IS
   BEGIN
      Ada.Text_IO.Put_Line("Usage: " &
         Ada.Command_Line.Command_Name & " [arguments]");
      Ada.Text_IO.Put_Line(" -h : print this message");
      Ada.Text_IO.Put_Line(" -l : remap the internet");
      Ada.Text_IO.Put_Line(" -s address : set upstream server IP");
      Ada.Text_IO.Put_Line(" -b blacklist: list of bad/good domains (regexs)");
      Ada.Text_IO.Put_Line(" -g whitelist: list of good domains (regexs), processed before blacklist");
      --Ada.Text_IO.Put_Line(" -c filename : records to add to cache");
      Ada.Text_IO.Put_Line(" -d : enable debugging messages");
   END Usage;

   PROCEDURE Process_Bad_Domains(Filename : IN String) IS
   BEGIN
      Check_Patterns.Initialize_Bad_Patterns(Filename);
   END Process_Bad_Domains;

   PROCEDURE Process_Good_Domains(Filename : IN String) IS
   BEGIN
      Check_Patterns.Initialize_Good_Patterns(Filename);
   END Process_Good_Domains;

--   PROCEDURE Add_To_Cache(Filename : IN String) IS
--   BEGIN
--      NULL;
--   END Add_To_Cache;

   PROCEDURE Process_Command_Line IS
      Arg_Num : Natural := 1;
   BEGIN
      --for now, any command line arg turns on debugging
      WHILE Arg_Num <= Ada.Command_Line.Argument_Count LOOP
         if ada.Command_Line.Argument(arg_num)="-d" then
            Global_Buffers.QueryBuffer.SetDebugging(true);
            Global_Buffers.ResponseBuffer.SetDebugging(True);
         ELSIF Ada.Command_Line.Argument(Arg_Num)="-l" THEN
            flags.remap := true;
         ELSIF Ada.Command_Line.Argument(Arg_Num)="-h" THEN
            Usage;
            Gnat.Os_Lib.Os_Exit(0);
         ELSIF Ada.Command_Line.Argument(Arg_Num)="-s" THEN
            Arg_Num := Arg_Num + 1;
            IF Arg_Num <= Ada.Command_Line.Argument_Count THEN
               IP_To_Be_Used := new String'(ada.Command_Line.Argument(arg_num));
            ELSE
               Usage;
               Gnat.Os_Lib.Os_Exit(0);
            end if;
         ELSIF Ada.Command_Line.Argument(Arg_Num)="-b" THEN
            Arg_Num := Arg_Num + 1;
            IF Arg_Num <= Ada.Command_Line.Argument_Count THEN
               Process_Bad_Domains(ada.Command_Line.Argument(arg_num));
            ELSE
               Usage;
               Gnat.Os_Lib.Os_Exit(0);
            end if;
         ELSIF Ada.Command_Line.Argument(Arg_Num)="-g" THEN
            Arg_Num := Arg_Num + 1;
            IF Arg_Num <= Ada.Command_Line.Argument_Count THEN
               Process_Good_Domains(ada.Command_Line.Argument(arg_num));
            ELSE
               Usage;
               Gnat.Os_Lib.Os_Exit(0);
            END IF;
--         ELSIF Ada.Command_Line.Argument(Arg_Num)="-c" THEN
--            Arg_Num := Arg_Num + 1;
--            IF Arg_Num <= Ada.Command_Line.Argument_Count THEN
--               Add_To_Cache(ada.Command_Line.Argument(arg_num));
--            ELSE
--               Usage;
--               Gnat.Os_Lib.Os_Exit(0);
--            end if;
         END IF;
         Arg_Num := Arg_Num + 1;
      END loop;

   end Process_Command_Line;

end Non_Spark_Command_Line_Pkg;
