SEPARATE(Process_Dns_Request)
PROCEDURE Create_Response (
      Input_Message  : IN     DNS_Types.Dns_Message;
      Input_Bytes    : IN     DNS_Types.Message_Length_Range;
      Output_Message : IN OUT DNS_Types.Dns_Message;
      Output_Bytes   :    OUT DNS_Types.Message_Length_Range;
      Max_Transmit   :    OUT DNS_Types.Message_Length_Range;
      Source_Address : IN     DNS_Network.Network_Address_And_Port;
      Domainname_Out :    OUT RR_Type.WireStringType) IS
   Start_Byte       : DNS_Types.Message_Bytes_Range;
   Query_End_Byte   : DNS_Types.Message_Bytes_Range;
   Domainname       : RR_Type.WireStringType;
   Query_Type       : DNS_Types.Query_Type;
   Query_Class      : DNS_Types.Query_Class;
   CNAMEChainLength : DNS_Types.Unsigned_Short;
   NumFound         : Rr_Type.NumberOfRecordsType;
   Counter          : Natural;
   Qname_Location   : DNS_Types.QNAME_PTR_RANGE;
   Qname_Locations  : QNAME_PTR_RANGE_Array;
   NS_Replies       : RR_Type.Ns_Record_Type.NSRecordBucketType;
   MX_Replies       : RR_Type.MX_Record_Type.MXRecordBucketType;
   Answer_Count     : DNS_Types.Unsigned_Short;
   Additional_Count : DNS_Types.Unsigned_Short;
   DNSSEC           : Boolean;

   InTable : Boolean;

   FirstLoc        : Integer                := 1;
   SecondLoc       : Integer;
   ThirdLoc        : Integer;
   ShortenedDomain : Rr_Type.WireStringType := Rr_Type.WireStringType'(OTHERS => ' ');
   PerDomain       : Rr_Type.WireStringType; --replaced length chars with periods, for log readability

   --these nested procedures reduce the complexity of create_response and
   --so make the simplifier run more efficiently in response to code changes
   PROCEDURE Process_Header_And_Question_Section (
         Input_Message  : IN     DNS_Types.DNS_Message;
         Output_Message : IN OUT DNS_Types.DNS_Message;
         Domainname     :    OUT RR_Type.WireStringType;
         Query_Type     :    OUT DNS_Types.Query_Type;
         Query_Class    :    OUT DNS_Types.Query_Class;
         Query_End_Byte :    OUT DNS_Types.Message_Bytes_Range)
      --# global in out Protected_SPARK_IO_05.SPARK_IO_PO;
      --# derives Domainname, Query_Type, Query_Class, Query_End_Byte from Input_Message &
      --#    Output_Message from Input_Message, Output_Message &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from Protected_SPARK_IO_05.SPARK_IO_PO, Input_Message;
   IS
   BEGIN
      -- I would like to only copy the header here, but I get a flow error!!
      --# accept Flow, 10, Output_message.Header.QR,
      --#   "The rest of the header fields retain their newly assigned values";
      --# accept Flow, 10, Output_message.Header.ANCOUNT,
      --#   "The rest of the header fields retain their newly assigned values";
      --# accept Flow, 10, Output_message.Header.AA,
      --#   "The rest of the header fields retain their newly assigned values";
      Output_Message.Header := Input_Message.Header;
      Output_Message.Header.AA := False;
      --# end accept;
      --# end accept;
      --# end accept;
      Output_Message.Header.QR := True;
      -- Keep # of queries and send the query back!! (i.e. don't do line below)
      -- Response_Header.QDCOUNT := 0;

      Get_Query_Name_Type_Class(Input_Message, Domainname,
         Query_Type, Query_Class, Query_End_Byte);

      --copy the question section from the input message
      FOR I IN DNS_Types.Message_Bytes_Range RANGE 1..Query_End_Byte LOOP
         --# assert Query_End_Byte >= 4 and Query_End_Byte <= dns_types.message_bytes_range'last;
         Output_Message.Bytes(I) := Input_Message.Bytes(I);
      END LOOP;

      -- we start out with no responses
      Output_Message.Header.ANCount := 0;
   END Process_Header_And_Question_Section;

   PROCEDURE Walk_CNAME_Chain (
         Domainname       : IN OUT RR_Type.WireStringType;
         Start_Byte       : IN OUT DNS_Types.Message_Bytes_Range;
         Answer_Count     : IN OUT DNS_Types.Unsigned_Short;
         Output_Message   : IN OUT DNS_Types.DNS_Message;
         CNAMEChainLength :    OUT DNS_Types.Unsigned_Short;
         Qname_Location   :    OUT DNS_Types.QNAME_PTR_RANGE)
      --# global in DNS_Table_Pkg.DNS_Table, Ada.Real_Time.ClockTime;
      --#    in out Protected_SPARK_IO_05.SPARK_IO_PO;
      --# derives Domainname from Domainname, Ada.Real_Time.ClockTime,
      --#       DNS_Table_Pkg.DNS_Table &
      --#    Start_Byte from Start_Byte, Domainname, Ada.Real_Time.ClockTime,
      --#       DNS_Table_Pkg.DNS_Table &
      --#    Answer_Count from Answer_Count, Domainname, Ada.Real_Time.ClockTime,
      --#       DNS_Table_Pkg.DNS_Table &
      --#    Output_Message from Output_Message, Start_Byte, Domainname, Ada.Real_Time.ClockTime,
      --#       DNS_Table_Pkg.DNS_Table &
      --#    CNAMEChainLength from Domainname, Ada.Real_Time.ClockTime, DNS_Table_Pkg.DNS_Table &
      --#    Qname_Location from Start_Byte, Domainname, Ada.Real_Time.ClockTime,
      --#       DNS_Table_Pkg.DNS_Table &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from Protected_SPARK_IO_05.SPARK_IO_PO,
      --#       Ada.Real_Time.ClockTime, DNS_Table_Pkg.DNS_Table, Domainname;
   IS
      ReturnedCNAMERecords : Rr_Type.CNAME_Record_Type.CNAMERecordBucketType;
      NumFound             : Rr_Type.NumberOfRecordsType;
      Tmp                  : Rr_Type.DomainNameStringType;
      Ignored              : Boolean;
      Output_Bytes         : Dns_Types.Message_Length_Range;
   BEGIN
      Qname_Location := 12;
      CNAMEChainLength := 0;
      --check to see if domain name aliased with CNAME rec
      DNS_Table_Pkg.DNS_Table.QueryCNAMERecords(
         DomainName      => Domainname,
         ReturnedRecords => ReturnedCNAMERecords,
         HowMany         => NumFound);

      --this should never happen, but it is a useful consistency check
      --if this prints, there is an error/inconsistency in how IRONSIDES
      --manages the DNS cache
      IF NumFound > 1 THEN
         Rr_Type.ConvertWireToDomainName(DomainName,Tmp,Ignored);
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            "Warning:  More than one cname record for domain ", 0);
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output, Tmp, 0);
         Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         FOR I IN Natural RANGE 1..NumFound LOOP
            --# assert I >= rr_type.NumberOfRecordsType'first and
            --#    I <= rr_type.NumberOfRecordsType'last;
            Rr_Type.ConvertWireToDomainName(ReturnedCNAMERecords(I).CanonicalDomainName,
               Tmp,Ignored);
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,Tmp, 0);
            Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         END LOOP;
         IF Ignored = Ignored THEN --make flow error go away
            Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
            Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         END IF;
      END IF;

      --if there was a CNAME record, follow the chain
      IF NumFound >= 1 THEN
         CNAMEChainLength := 1;
         WHILE (CNAMEChainLength <= MaxCNAMEChainLength) AND (NumFound > 0) LOOP
            --# assert CNAMEChainLength >= 1 and
            --#    CNAMEChainLength <= MAXCNAMEChainLength;
            Process_Response_Cname(
               Start_Byte     => Start_Byte,
               Cnames         => ReturnedCNAMERecords,
               Domainname     => Domainname,
               Qname_Location => Qname_Location,
               Output_Message => Output_Message,
               Answer_Count   => Answer_Count,
               Output_Bytes   => Output_Bytes);
            CNAMEChainLength := CNAMEChainLength+1;
            Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
               DNS_Types.Header_Bits/8);
            DNS_Table_Pkg.DNS_Table.QueryCNAMERecords(
               DomainName      => Domainname,
               ReturnedRecords => ReturnedCNAMERecords,
               HowMany         => NumFound);

            --this should never happen, but it is a useful consistency check
            --if this prints, there is an error/inconsistency in how IRONSIDES
            --manages the DNS cache
            IF NumFound > 1 THEN
               Rr_Type.ConvertWireToDomainName(DomainName,Tmp,Ignored);
               Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                  "Warning:  More than one cname record for domain ", 0);
               Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output, Tmp, 0);
               Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
               FOR I IN Natural RANGE 1..NumFound LOOP
                  --# assert I >= rr_type.NumberOfRecordsType'first and
                  --#    I <= rr_type.NumberOfRecordsType'last and
                  --#    CNAMEChainLength >= 1;
                  Rr_Type.ConvertWireToDomainName(ReturnedCNAMERecords(I).CanonicalDomainName,
                     Tmp,Ignored);
                  Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,Tmp, 0);
                  Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
               END LOOP;
               IF Ignored = Ignored THEN --make flow error go away
                  Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
                  Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
               END IF;
            END IF; -- >1 CNAME record for same name, error
         END LOOP; --traversing CNAME record chain
      END IF; -- CNAME record found for domain being queried

      IF CNAMEChainLength > MaxCNAMEChainLength AND NumFound > 0 THEN
         Rr_Type.ConvertWireToDomainName(DomainName,Tmp,Ignored);
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            "Warning:  CNAME chain ending in domain ", 0);
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output, Tmp, 0);
         Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            "is too long; further aliases not returned.", 0);
         Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            "To fix, increase MaxCNAMEChainLength in process_dns_request.ads", 0);
         IF Ignored = Ignored THEN --make flow error go away
            Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         END IF;
      END IF;
   END Walk_CNAME_Chain;

BEGIN --Create_Response()
   Process_Header_And_Question_Section(
      Input_Message  => Input_Message,
      Output_Message => Output_Message,
      Domainname     => Domainname,
      Query_Type     => Query_Type,
      Query_Class    => Query_Class,
      Query_End_Byte => Query_End_Byte);
   Start_Byte := Query_End_Byte;

   -- we start out with no responses
   Answer_Count := 0;

   --if there are any CNAME records for this domain, process those first
   Walk_CNAME_Chain(
      DomainName       => DomainName,
      Start_Byte       => Start_Byte,
      Answer_Count     => Answer_Count,
      Output_Message   => Output_Message,
      CNAMEChainLength => CNAMEChainLength,
      Qname_Location   => Qname_Location);

   --Can probably replace the need for this assertion with a postcondition on
   --Walk_CNAME_Chain at some point
   IF Answer_Count > MaxCNAMEChainLength THEN
      Answer_Count := MaxCNAMEChainLength;
   END IF;

   -- Process Domain (lowercase and remove first subdomains)
   PerDomain := Domainname;
   PerDomain(FirstLoc) := '.';
   ThirdLoc := (FirstLoc + Character'Pos(Domainname(FirstLoc))) + 1;
   SecondLoc := FirstLoc;
   FindEnd:
   while thirdloc<=domainname'last LOOP
         --# assert FirstLoc>=1 and FirstLoc<=domainname'last and
         --# thirdloc>=1 and thirdloc<=domainname'last and
         --# answer_count>=0 and answer_count<=maxcnamechainlength;
      SecondLoc := FirstLoc;
      FirstLoc := ThirdLoc;
      PerDomain(FirstLoc) := '.';
      ThirdLoc := (FirstLoc + 1) + Character'Pos(Domainname(FirstLoc));
      EXIT FindEnd WHEN ThirdLoc>Domainname'Last OR ELSE
         Character'Pos(Domainname(ThirdLoc)) = 0;
   END LOOP FindEnd;
   FOR I IN Natural RANGE 1..Domainname'Last-SecondLoc LOOP
      --# assert answer_count>=0 and answer_count<=maxcnamechainlength and
      --# secondloc = secondloc% and
      --# secondloc>=1 and secondloc+i<=domainname'last;
      ShortenedDomain(I) := Domainname(SecondLoc+I);
   END LOOP;
   --shortenedDomain(1..Domainname'Last-secondLoc) := Domainname(secondLoc+1..Domainname'Last);
   ShortenedDomain := To_Lower(ShortenedDomain);
   Dns_Table_Pkg.DNS_Table.QueryCorruptRecords(
      DomainName => ShortenedDomain,
      InTable    => InTable);
   --Branch here. Check if invalid
   IF(InTable) THEN --determined to be invalid
      Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
         Protected_SPARK_IO_05.Spark_IO_PO.Standard_Output,
         "Request Blocked: ",0);
      Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
         Protected_SPARK_IO_05.Spark_IO_PO.Standard_Output,
         PerDomain,32);
      Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
         Protected_SPARK_IO_05.Spark_IO_PO.Standard_Output,
         "From: ",0);
      Dns_Network.Print_Server_Info(Source_Address);
      Create_Response_Zero(
         Start_Byte     => Start_Byte,
         Domainname     => Domainname,
         Qname_Location => Qname_Location,
         Output_Message => Output_Message,
         Answer_Count   => Answer_Count,
         Output_Bytes   => Output_Bytes);
      Additional_Count := 0;
      CNameChainLength := 0;
   ELSE
      Additional_Count := 0;
      IF (Query_Class /= DNS_Types.IN_CLASS AND Query_Class /= DNS_Types.ANY_CLASS) OR
            (Flags.Remap AND (Query_Type=Dns_Types.SOA OR query_type=dns_types.ns or
               Query_Type=Dns_Types.dnskey or query_type=dns_types.NSEC or query_type=dns_types.RRSIG)) THEN
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_Line(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            "bad query class:  " & dns_types.query_type'image(query_type), 0);
         Create_Response_Error(
            Input_Bytes    => Input_Bytes,
            Output_Message => Output_Message,
            Output_Bytes   => Output_Bytes);
         CnameChainLength := 0;
         Answer_Count := 0;
         --Cached responses to ANY queries appear to be incorrectly formatted,
         --dig complains of a "bad label type".  Need to fix this.
      ELSIF Query_Type = DNS_Types.ANY THEN
         Create_Response_A(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);
         Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
            DNS_Types.Header_Bits/8);
         --if not flags.remap then
            Create_Response_AAAA(
               Start_Byte     => Start_Byte,
               Domainname     => Domainname,
               Qname_Location => Qname_Location,
               Output_Message => Output_Message,
               Answer_Count   => Answer_Count,
               Output_Bytes   => Output_Bytes);
            Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
                  DNS_Types.Header_Bits/8);
         --end if;
         --# accept Flow, 10, MX_Replies, "not needed for ANY query";
         --# accept Flow, 10, Qname_Locations, "not needed for ANY query";
         --# accept Flow, 10, NumFound, "not needed for any query";
         Create_Response_MX(
            Start_Byte      => Start_Byte,
            Domainname      => Domainname,
            Num_Found       => NumFound,
            Qname_Location  => Qname_Location,
            Qname_Locations => Qname_Locations,
            Replies         => MX_Replies,
            Output_Message  => Output_Message,
            Answer_Count    => Answer_Count,
            Output_Bytes    => Output_Bytes);
         --# end accept;
         --# end accept;
         --# end accept;
         Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
            DNS_Types.Header_Bits/8);
         if not flags.remap then
         --# accept Flow, 10, NS_Replies, "not needed for ANY query";
         --# accept Flow, 10, Qname_Locations, "not needed for ANY query";
         --# accept Flow, 10, NumFound, "not needed for any query";
         Create_Response_NS(
            Start_Byte      => Start_Byte,
            Domainname      => Domainname,
            Num_Found       => NumFound,
            Qname_Location  => Qname_Location,
            Qname_Locations => Qname_Locations,
            Replies         => NS_Replies,
            Output_Message  => Output_Message,
            Answer_Count    => Answer_Count,
            Output_Bytes    => Output_Bytes);
         --# end accept;
         --# end accept;
         --# end accept;
         Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
            DNS_Types.Header_Bits/8);
         end if;
         Create_Response_PTR(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);
         Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
            DNS_Types.Header_Bits/8);
         if not flags.remap then
         Create_Response_SOA(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);
         Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) - DNS_Types.Header_Bits/8);
         Create_Response_DNSKEY(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);
         --         This call causes flow errors because certain out parameter
         --         values are not consumed.  Need to fix how ANY queries are
         --         handled, fix this then.
         --         Start_Byte := DNS_Types.message_bytes_range(Integer(Output_Bytes) -
         --            DNS_Types.Header_Bits/8);
         --         Create_Response_NS(
         --            Start_Byte     => Start_Byte,
         --            Domainname     => Domainname,
         --            Num_Found       => NumFound,
         --            Qname_Location  => Qname_Location,
         --            Qname_Locations => Qname_Locations,
         --            Replies         => NS_Replies,
         --            Output_message   => Output_message,
         --            Answer_Count    => Answer_Count,
         --            Output_Bytes    => Output_Bytes);
         Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
            DNS_Types.Header_Bits/8);
         Create_Response_NSEC(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes); --DNS_Types.ANY
         end if;
      ELSIF Query_Type = DNS_Types.SOA THEN
         Create_Response_SOA(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);

      ELSIF Query_Type = DNS_Types.PTR THEN
         Create_Response_PTR(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);

      ELSIF Query_Type = DNS_Types.A THEN
         Create_Response_A(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);

      ELSIF Query_Type = DNS_Types.AAAA THEN
         Create_Response_AAAA(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);

         --needed to add this because with caching, can have a CNAME query for which
         --DNS table lookup above for CNAME records can come up empty.
      ELSIF Query_Type = DNS_Types.CNAME THEN
         --these make flow errors go away, required for heap overflow assertion below
         Output_Bytes := DNS_Types.Header_Bits/8+1;
         Additional_Count := 0;
         Answer_Count := 0;
         --don't need to do anything else

      ELSIF Query_Type = DNS_Types.DNSKEY THEN
         Create_Response_DNSKEY(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);

      ELSIF Query_Type = DNS_Types.NSEC THEN
         Create_Response_NSEC(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);

      ELSIF Query_Type = DNS_Types.RRSIG THEN
         Create_Response_RRSIG(
            Start_Byte     => Start_Byte,
            Domainname     => Domainname,
            Qname_Location => Qname_Location,
            Output_Message => Output_Message,
            Answer_Count   => Answer_Count,
            Output_Bytes   => Output_Bytes);

      ELSIF Query_Type = DNS_Types.MX THEN
         Create_Response_MX(
            Start_Byte      => Start_Byte,
            Domainname      => Domainname,
            Num_Found       => NumFound,
            Qname_Location  => Qname_Location,
            Qname_Locations => Qname_Locations,
            Replies         => MX_Replies,
            Output_Message  => Output_Message,
            Answer_Count    => Answer_Count,
            Output_Bytes    => Output_Bytes);
         Counter := 1;
         Additional_Count := 0;
         WHILE Counter <= NumFound AND Additional_Count < DNS_Types.Unsigned_Short'Last-DNS_Types.Unsigned_Short(
               2*Rr_Type.MaxNumRecords) LOOP
            --# assert Counter >= 1 and Counter<=NumFound and
            --#    Answer_Count >=0 and Answer_Count <= 65535 and
            --#    Additional_Count >= 0 and
            --#    (for all Z in RR_Type.ReturnedRecordsIndexType =>
            --#       (Qname_Locations(Z) >= 0 and Qname_Locations(Z) < 16384)) and
            --#    Qname_Location >=0 and Qname_Location <= 16383 and
            --#    Additional_Count < DNS_Types.Unsigned_Short'Last-DNS_Types.Unsigned_Short(
            --#    2*Rr_Type.MaxNumRecords) and
            --#    NumFound >= 0 and NumFound <= rr_type.MaxNumRecords and
            --#    Integer(Output_Bytes) >= DNS_Types.Header_Bits/8+1 and
            --#    Integer(Output_Bytes) <= DNS_Types.Message_Size;
            Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
               DNS_Types.Header_Bits/8);
            Create_Response_A(
               Start_Byte     => Start_Byte,
               Domainname     => MX_Replies (Counter).MailExchanger,
               Qname_Location => Qname_Locations (Counter),
               Output_Message => Output_Message,
               Answer_Count   => Additional_Count,
               Output_Bytes   => Output_Bytes);
            Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
               DNS_Types.Header_Bits/8);
            Create_Response_AAAA(
               Start_Byte     => Start_Byte,
               Domainname     => MX_Replies (Counter).MailExchanger,
               Qname_Location => Qname_Locations (Counter),
               Output_Message => Output_Message,
               Answer_Count   => Additional_Count,
               Output_Bytes   => Output_Bytes);
            Counter := Counter + 1;
         END LOOP; --DNS_Types.MX

      ELSIF Query_Type = DNS_Types.NS THEN
         Create_Response_NS(
            Start_Byte      => Start_Byte,
            Domainname      => Domainname,
            Num_Found       => NumFound,
            Qname_Location  => Qname_Location,
            Qname_Locations => Qname_Locations,
            Replies         => NS_Replies,
            Output_Message  => Output_Message,
            Answer_Count    => Answer_Count,
            Output_Bytes    => Output_Bytes);
         Counter := 1;
         Additional_Count := 0;
         WHILE Counter <= NumFound AND Additional_Count < DNS_Types.Unsigned_Short'Last-DNS_Types.Unsigned_Short(
               2*Rr_Type.MaxNumRecords) LOOP
            --# assert Counter >= 1 and Counter<=NumFound and
            --#    (for all Z in RR_Type.ReturnedRecordsIndexType =>
            --#       (Qname_Locations(Z) >= 0 and Qname_Locations(Z) < 16384)) and
            --#    Qname_Location >=0 and Qname_Location <= 16383 and
            --#    Answer_Count >=0 and Answer_Count <= 65535 and
            --#    Additional_Count >= 0 and
            --#    Additional_Count < DNS_Types.Unsigned_Short'Last-DNS_Types.Unsigned_Short(
            --#    2*Rr_Type.MaxNumRecords) and
            --#    NumFound >= 0 and NumFound <= rr_type.MaxNumRecords and
            --#    Integer(Output_Bytes) >= DNS_Types.Header_Bits/8+1 and
            --#    Integer(Output_Bytes) <= DNS_Types.Message_Size;
            Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
               DNS_Types.Header_Bits/8);
            Create_Response_A(
               Start_Byte     => Start_Byte,
               Domainname     => NS_Replies (Counter).Nameserver,
               Qname_Location => Qname_Locations (Counter),
               Output_Message => Output_Message,
               Answer_Count   => Additional_Count,
               Output_Bytes   => Output_Bytes);
            Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
               DNS_Types.Header_Bits/8);
            Create_Response_AAAA(
               Start_Byte     => Start_Byte,
               Domainname     => NS_Replies (Counter).Nameserver,
               Qname_Location => Qname_Locations (Counter),
               Output_Message => Output_Message,
               Answer_Count   => Additional_Count,
               Output_Bytes   => Output_Bytes);
            Counter := Counter + 1;
         END LOOP;  --DNS_Types.NS

      ELSE
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            "bad/unsupported query type  ", 0);
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            Dns_Types.Query_Type'Pos(Query_Type), 2, 10);
         Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         Create_Response_Error(
            Input_Bytes    => Input_Bytes,
            Output_Message => Output_Message,
            Output_Bytes   => Output_Bytes);
      END IF;

      -- this assert helps the simplifier run faster
      -- (see postconditions in procedure spec)
      --# assert
      --#    Answer_Count >=0 and Answer_Count <= 65535 and
      --#    Additional_Count >= 0 and
      --#    Integer(Output_Bytes) >= DNS_Types.Header_Bits/8+1 and
      --#    Integer(Output_Bytes) <= DNS_Types.Message_Size;
   END IF;

   DNSSEC := False;
   Max_Transmit := DNS_Types.UDP_Max_Size;
   -- Handle EDNS additional OPT record here!
   IF Input_Message.Header.QDCount = 1 AND
         Input_Message.Header.ARCount = 1 AND
         Additional_Count < DNS_Types.Unsigned_Short'Last THEN
      Start_Byte := DNS_Types.Message_Bytes_Range(Integer(Output_Bytes) -
         DNS_Types.Header_Bits/8);
      Create_Response_EDNS(
         Input_Message    => Input_Message,
         Input_Bytes      => Input_Bytes,
         Query_End_Byte   => Query_End_Byte,
         Start_Byte       => Start_Byte,
         Output_Message   => Output_Message,
         Output_Bytes     => Output_Bytes,
         Additional_Count => Additional_Count,
         DNSSEC           => DNSSEC,
         Max_Transmit     => Max_Transmit);
   ELSIF Input_Message.Header.QDCount /= 1 THEN
      Protected_SPARK_IO_05.SPARK_IO_PO.Put_Line(Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,"query count > 1", 0);
   ELSIF Input_Message.Header.ARCount > 1 THEN
      Protected_SPARK_IO_05.SPARK_IO_PO.Put_Line(Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,"ar count > 1", 0);
   END IF;



   -- this assert helps the simplifier run faster
   -- (see postconditions in procedure spec)
   --# assert
   --#    Answer_Count >=0 and Answer_Count <= 65535 and
   --#    Additional_Count >= 0 and
   --#    Integer(Max_Transmit) <= DNS_Types.Message_Size and Max_Transmit >= DNS_Types.UDP_Max_Size and
   --#    Integer(Output_Bytes) >= DNS_Types.Header_Bits/8+1 and
   --#    Integer(Output_Bytes) <= DNS_Types.Message_Size;

   IF DNSSEC AND Answer_Count > 0 THEN
      Output_Bytes := (Output_Bytes-1)+1;
   END IF;
   --If the only records that come back are in a CNAME chain,
   --(presumably because the non-CNAME records that would normally
   --end it have expired), that counts as having no answer at all.
   --Setting Answer_Count to zero will cause the query to be sent
   --upstream.
   IF Answer_Count = CNAMEChainLength-1 THEN
      Answer_Count := 0;
   END IF;
   Output_Message.Header.ANCount := Answer_Count;
   Output_Message.Header.ARCount := Additional_Count;
   IF Answer_Count > 0 THEN
      -- our answer is non-authoritative
      Output_Message.Header.AA := False;
      Output_Message.Header.RCODE := DNS_Types.No_Error;
      -- explicit check for conditions below helps SPARK
   ELSIF Answer_Count=0 AND
         Start_Byte+12 <= DNS_Types.Message_Bytes_Range'Last THEN
      --check for cached NXDOMAIN records
      Create_Response_NXDOMAIN(
         Start_Byte     => Start_Byte,
         Domainname     => Domainname,
         Qname_Location => Qname_Location,
         Output_Message => Output_Message,
         Answer_Count   => Answer_Count,
         Output_Bytes   => Output_Bytes);
      --need to create AUTHORITY section here, update header?
      IF Answer_Count > 0 THEN
         -- our answer is non-authoritative
         Output_Message.Header.AA := False;
         Output_Message.Header.RCODE := DNS_Types.NAME_ERROR;
         Output_Message.Header.ANCount := Answer_Count;
      END IF;
   ELSE --Answer_Count < 0
      NULL;  --included for readability, will never be executed
   END IF;
   --Ada.Text_IO.Put_Line("creaTE RESPONSE end!" & dns_types.unsigned_short'image(Output_message.Header.ANCount) & "::" & dns_types.message_length_range'image(output_bytes));
   Domainname_Out := PerDomain;
END Create_Response;

