SEPARATE(Process_Dns_Request)
   -- if we get in an EDNS option, we will add a response in kind
   procedure Create_Response_EDNS(
         Input_message      : in DNS_Types.dns_message;
         Input_Bytes        : in DNS_Types.message_length_range;
         Query_End_Byte     : in DNS_Types.message_bytes_range;
         Start_Byte         : in DNS_Types.message_bytes_range;
         Output_message     : in out DNS_Types.dns_message;
         Output_Bytes       : out DNS_Types.message_length_range;
         Additional_Count   : in out DNS_Types.Unsigned_Short;
         DNSSEC             : out Boolean;
         Max_Transmit       : OUT DNS_Types.Message_Length_Range)
      is
      EDNS_Rec : DNS_Types.EDNS_Record;
      function To_Query_Type is new Ada.Unchecked_Conversion(DNS_Types.Unsigned_Short,DNS_Types.Query_Type);
      function From_Query_Type is new Ada.Unchecked_Conversion(DNS_Types.Query_Type,DNS_Types.Unsigned_Short);
   begin
      Max_Transmit := DNS_Types.UDP_Max_Size;
      Output_Bytes := DNS_Types.message_length_range(Start_Byte + DNS_Types.message_bytes_range(DNS_Types.Header_Bits/8));
      DNSSEC := False;

      -- if there's more to the message, check for the OPT code
      if (Integer(Query_End_Byte)+11)+(DNS_Types.Header_Bits/8)<=Integer(Input_Bytes) and
         (Integer(Start_Byte)+11)+(DNS_Types.Header_Bits/8)<DNS_Types.Message_Size then
         EDNS_Rec.Root    := Character'Val(Input_message.Bytes(Query_End_Byte+1));
         --# accept Warning, 12, To_Query_Type, "unchecked conversions ok";
         EDNS_Rec.Code    := To_Query_Type(DNS_Types.Unsigned_Short(
            Input_message.Bytes(Query_End_Byte+2))*256+
            DNS_Types.Unsigned_Short(Input_message.Bytes(Query_End_Byte+3)));
         if EDNS_Rec.Root = Character'Val(0) and
            EDNS_Rec.Code = DNS_Types.OPT then
            EDNS_Rec.Payload_Size := DNS_Types.Unsigned_Short(Input_message.Bytes(
               Query_End_Byte+4))*256+
               DNS_Types.Unsigned_Short(Input_message.Bytes(Query_End_Byte+5));
            --EDNS_Rec.RCODE   := Input_message.Bytes(Query_End_Byte+6);
            --EDNS_Rec.Version := Input_message.Bytes(Query_End_Byte+7);
            EDNS_Rec.ZTop    := Input_message.Bytes(Query_End_Byte+8);
            --EDNS_Rec.ZBottom := Input_message.Bytes(Query_End_Byte+9);
            --EDNS_Rec.RDLEN    := DNS_Types.Unsigned_Short(
            --   Input_message.Bytes(Query_End_Byte+10))*256+
            --   DNS_Types.Unsigned_Short(Input_message.Bytes(Query_End_Byte+11));

            -- when we respond, we'll respond with AT LEAST UDP_Max_Size,
            -- but no bigger than our Message_Size
            Max_Transmit := DNS_Types.message_length_range(
               DNS_Types.Unsigned_Short'Min(DNS_Types.Message_Size,
                  EDNS_Rec.Payload_Size));
            Max_Transmit := DNS_Types.message_length_range'Max(
               DNS_Types.UDP_Max_Size, Max_Transmit);

            Output_message.Bytes(Start_Byte+1) := 0;
            -- high order byte of OPT is 0
            Output_message.Bytes(Start_Byte+2) := 0;
            -- the mod here is superfluous as OPT = 41
            --# accept Warning, 12, From_Query_Type, "unchecked conversions ok";
            Output_message.Bytes(Start_Byte+3) := DNS_Types.Byte(
               From_Query_Type(DNS_Types.OPT) mod 256);

            -- split unsigned short into two bytes
            Output_message.Bytes(Start_Byte+4) := DNS_Types.Byte(
               Max_Transmit/256);
            Output_message.Bytes(Start_Byte+5) := DNS_Types.Byte(
               Max_Transmit mod 256);

            Output_message.Bytes(Start_Byte+6) := 0;
            Output_message.Bytes(Start_Byte+7) := 0;
            -- FLAGS (DNSSEC ONLY), see RFC 3225
            if (EDNS_Rec.ZTop and Dns_Types.DNSSECMASK) /= 0 then
               Output_message.Bytes(Start_Byte+8) := Dns_Types.DNSSECMASK;
               DNSSEC := True;
            else
               Output_message.Bytes(Start_Byte+8) := 0;
            end if;
            Output_message.Bytes(Start_Byte+9) := 0;
            -- RDLEN = 0
            Output_message.Bytes(Start_Byte+10) := 0;
            Output_message.Bytes(Start_Byte+11) := 0;

            Additional_Count := Additional_Count + 1;
            Output_Bytes := DNS_Types.message_length_range((Start_Byte + 11) +
               DNS_Types.message_bytes_range(DNS_Types.Header_Bits/8));
         end if;
      end if;
   end Create_Response_EDNS;

