SEPARATE(Process_Dns_Request)
   procedure Create_Response_MX(
         Start_Byte      : in DNS_Types.message_bytes_range;
         Domainname      : in RR_Type.WireStringType;
         Qname_Location  : IN DNS_Types.QNAME_PTR_RANGE;
         output_message  : in out DNS_Types.DNS_Message;
         Answer_Count    : in out DNS_Types.Unsigned_Short;
         Output_Bytes    : OUT DNS_Types.Message_Length_Range;
         Num_Found       : out RR_Type.NumberOfRecordsType;
         Qname_Locations : out QNAME_PTR_RANGE_Array;
         Replies         : out RR_Type.mx_record_type.MXRecordBucketType) is
      Response_Counter : Natural;
      Current_Byte  : DNS_Types.Message_Bytes_Range;
      Current_Name_Length : RR_Type.WireStringTypeIndex;
   begin
      Qname_Locations := QNAME_PTR_RANGE_Array'(others => 0);
      Current_Byte := Start_Byte;
      -- accept Flow, 23, Replies, "it will fill in enough";
      DNS_Table_Pkg.DNS_Table.QueryMXRecords(
         DomainName      => Domainname,
         ReturnedRecords => Replies,
         HowMany         => Num_Found);
      -- end accept;
      if Num_Found>=1 then
         Response_Counter := 1;
         Current_Name_Length := RR_Type.WireNameLength(Replies(Response_Counter).mailExchanger);
         while Response_Counter <= Num_Found and then
            Integer(Current_Byte) < (DNS_Types.message_size-(14+DNS_Types.Header_Bits/8))-
               (Current_Name_Length) loop
            --# assert Response_Counter >=1 and Response_Counter <= Num_Found
            --# and current_name_length >=1 and current_name_length<=rr_type.WireStringTypeIndex'last
            --# and Num_Found <= rr_type.MaxNumRecords
            --# and Integer(Start_Byte) <= DNS_Types.Message_Size
            --# and Answer_Count = Answer_Count~
            --# and Answer_Count <= DNS_Types.Unsigned_Short'Last-DNS_types.Unsigned_Short(rr_type.MaxNumRecords)
            --# and Integer(Current_Byte) < (DNS_Types.Message_Size-(14+DNS_Types.Header_Bits/8))-
            --#    (Current_Name_Length)
            --# and Current_Byte >= 0;

            -- PTR to character of message
            Set_Unsigned_16(output_message.Bytes,Current_Byte+1,Unsigned_Types.Unsigned16(Qname_Location)+16#C000#);

            -- MX
            output_message.Bytes(Current_Byte+3) := 16#00#;
            output_message.Bytes(Current_Byte+4) := 16#0F#;

            -- IN
            output_message.Bytes(Current_Byte+5) := 16#00#;
            output_message.Bytes(Current_Byte+6) := 16#01#;

            Qname_Locations(Response_Counter) := DNS_Types.QNAME_PTR_RANGE(
               (Current_Byte+14)+ DNS_Types.Message_Bytes_Range(DNS_Types.Header_Bits/8));

            Set_TTL_Data_MX_Response(output_message.Bytes,Current_Byte+7,
               Replies(Response_Counter),Current_Name_Length);
            Response_Counter := Response_Counter + 1;
            Current_Byte := (Current_Byte + 14) + (DNS_Types.Message_Bytes_Range(Current_Name_Length)-1); -- minus 1 to remove null byte between entries
            if Response_Counter <= Num_Found then
               Current_Name_Length := RR_Type.WireNameLength(Replies(Response_Counter).mailExchanger);
            end if;
         end loop;
      end if;
      Output_Bytes := DNS_Types.Message_Length_Range(Current_Byte + DNS_Types.Message_Bytes_Range(DNS_Types.Header_Bits/8));
      Answer_Count := Answer_Count + DNS_Types.Unsigned_Short(Num_Found);
      -- accept Flow, 602, output_message, Replies,  "initialization is unneeded";
      -- accept Flow, 602, Output_Bytes, Replies,  "initialization is unneeded";
      -- accept Flow, 602, Replies, Replies,  "initialization is unneeded";
      -- accept Flow, 602, Qname_Locations, Replies,  "initialization is unneeded";
   end Create_Response_MX;
