SEPARATE(Process_Dns_Request)
   procedure Create_Response_NSEC(
         Start_Byte     : in DNS_Types.message_bytes_range;
         Domainname     : in RR_Type.WireStringType;
         Qname_Location : in DNS_Types.QNAME_PTR_RANGE;
         output_message  : in out DNS_Types.DNS_Message;
         Answer_Count   : in out DNS_Types.Unsigned_Short;
         Output_Bytes   : OUT DNS_Types.message_length_range) IS
      Current_Byte  : DNS_Types.message_bytes_range;
      ReturnedNSECRecords : Rr_Type.NSEC_Record_Type.NSECRecordBucketType;
      NumFound : Natural;
      Response_Counter : Natural;
      CurrentRec : Rr_Type.NSEC_Record_Type.NSECRecordType;
      bytesUsedInRRData  : DNS_Types.message_length_range;
   BEGIN
      -- accept Flow, 23, ReturnedNSECRecords, "it will fill in enough";
      DNS_Table_Pkg.DNS_Table.QueryNSECRecords(DomainName => Domainname,
         ReturnedRecords => ReturnedNSECRecords, HowMany => NumFound);
      -- end accept;
      Current_Byte := Start_Byte; -- actually the last byte of the question section
      if NumFound>=1 then
         Response_Counter := 1;
         CurrentRec := ReturnedNSECRecords(Response_Counter);
         WHILE Response_Counter <= NumFound AND Integer(Current_Byte) <
           DNS_Types.message_size-((((12 + Rr_Type.NSEC_Record_Type.MaxRRDataLength)
           + rr_type.maxDomainNameLength)+1) + DNS_Types.Header_Bits/8) LOOP

            -- NAME (2 bytes, PTR to name)
            Set_Unsigned_16(output_message.Bytes,Current_Byte+1,
               Unsigned_Types.Unsigned16(Qname_Location)+16#C000#);

            -- TYPE (2 bytes, NSEC --> 47)
            output_message.Bytes(Current_Byte+3) := 16#00#;
            output_message.Bytes(Current_Byte+4) := 16#2f#;

            -- CLASS (2 bytes, IN --> 1)
            output_message.Bytes(Current_Byte+5) := 16#00#;
            output_message.Bytes(Current_Byte+6) := 16#01#;

            --set TTL (4 bytes), RDLENGTH (2 bytes) and RDATA (domainNameLength + rrListLength) fields
            Set_TTL_Data_NSEC_Response(output_message.Bytes,Current_Byte+7,CurrentRec,bytesUsedInRRData);

            --# assert Response_Counter >=1 and Response_Counter <= NumFound
            --# and Integer(Start_Byte) <= DNS_Types.message_size
            --# and Answer_Count = Answer_Count~
            --# and Answer_Count <= DNS_Types.Unsigned_Short'Last-DNS_types.Unsigned_Short(rr_type.MaxNumRecords)
            --# and Integer(Current_Byte) < DNS_Types.message_size-((12 + Rr_Type.NSEC_Record_Type.MaxRRDataLength)
            --#   + (rr_type.maxDomainNameLength+1) + DNS_Types.Header_Bits/8)
            --# and bytesUsedInRRData <= DNS_Types.message_length_range(Rr_Type.NSEC_Record_Type.MaxRRDataLength)
            --# and numfound <= rr_type.MaxNumRecords;
            Current_Byte := Current_Byte + DNS_Types.message_bytes_range(12 + bytesUsedInRRData);
            Response_Counter := Response_Counter + 1;
            if Response_Counter <= NumFound then
               CurrentRec := ReturnedNSECRecords(Response_Counter);
            END IF;
         END LOOP;
      end if; --NumFound >= 1
      Output_Bytes := DNS_Types.message_length_range(Current_Byte + DNS_Types.message_bytes_range(DNS_Types.Header_Bits/8));
      Answer_Count := Answer_Count + DNS_Types.Unsigned_Short(numFound);
      -- accept Flow, 602, output_message, ReturnedNSECRecords,  "initialization is unneeded";
   END Create_Response_NSEC;
