SEPARATE(Process_Dns_Request)
   procedure Create_Response_SOA(
         Start_Byte      : in DNS_Types.message_bytes_range;
         Domainname      : in RR_Type.WireStringType;
         Qname_Location  : in DNS_Types.QNAME_PTR_RANGE;
         output_message   : in out DNS_Types.DNS_Message;
         Answer_Count    : in out DNS_Types.Unsigned_Short;
         Output_Bytes    : out DNS_Types.message_length_range) is
      Response_Counter       : Natural;
      Current_Byte           : DNS_Types.Message_Bytes_Range;
      Nameserver_Name_Length : RR_Type.WireStringTypeIndex;
      Mailbox_Name_Length    : RR_Type.WireStringTypeIndex;
      Num_Found              : RR_Type.NumberOfRecordsType;
      Replies                : RR_Type.SOA_record_type.SOARecordBucketType;
   begin
      Current_Byte := Start_Byte;
      -- accept Flow, 23, Replies, "it will fill in enough";
      DNS_Table_Pkg.DNS_Table.QuerySOARecords(
         DomainName      => Domainname,
         ReturnedRecords => Replies,
         HowMany         => Num_Found);
      -- end accept;
      if Num_Found>=1 then
         Response_Counter := 1;
         Nameserver_Name_Length := RR_Type.WireNameLength(Replies(Response_Counter).nameserver);
         Mailbox_Name_Length := RR_Type.WireNameLength(Replies(Response_Counter).email);
         if Integer(Current_Byte) < (DNS_Types.message_size-(32+DNS_Types.Header_Bits/8))-
            (Nameserver_Name_Length+Mailbox_Name_Length) then

            -- PTR to character of message
            Set_Unsigned_16(output_message.Bytes,Current_Byte+1,Unsigned_Types.Unsigned16(Qname_Location)+16#C000#);

            -- SOA
            output_message.Bytes(Current_Byte+3) := 16#00#;
            output_message.Bytes(Current_Byte+4) := 16#06#;

            -- IN
            output_message.Bytes(Current_Byte+5) := 16#00#;
            output_message.Bytes(Current_Byte+6) := 16#01#;

            -- TTL, RDLENGTH, RDATA
            Set_TTL_Data_SOA_Response(output_message.Bytes,Current_Byte+7,
               Replies(Response_Counter),Nameserver_Name_Length,Mailbox_Name_Length);
            Current_Byte := (Current_Byte + 32) + DNS_Types.Message_Bytes_Range(
               Nameserver_Name_Length+Mailbox_name_Length);
         end if;
      end if;
      Output_Bytes := DNS_Types.Message_Length_Range(Current_Byte + DNS_Types.Message_Bytes_Range(DNS_Types.Header_Bits/8));
      Answer_Count := Answer_Count + DNS_Types.Unsigned_Short(Num_Found);
      -- accept Flow, 602, output_message, Replies,  "initialization is unneeded";
      -- accept Flow, 602, Output_Bytes, Replies,  "initialization is unneeded";
   end Create_Response_SOA;
