SEPARATE (Process_Dns_Request)
   procedure Process_Response_Cname(
         Start_Byte     : in DNS_Types.message_bytes_range;
         Cnames         : in RR_Type.cname_record_type.CNAMERecordBucketType;
         Domainname     : OUT RR_Type.WireStringType;
         Qname_Location : in out DNS_Types.QNAME_PTR_RANGE;
         Output_Message  : IN OUT DNS_Types.Dns_Message;
         Answer_Count    : in out DNS_Types.Unsigned_Short;
         Output_Bytes   : out DNS_Types.message_length_range) is
      Current_Byte  : DNS_Types.message_bytes_range;
      Name_Length : RR_Type.WireStringTypeIndex;
      I : RR_Type.WireStringTypeIndex;
   begin
      Current_Byte := Start_Byte;
      Domainname := Cnames(Cnames'First).CanonicalDomainName;
      Name_Length := RR_Type.WirenameLength(Cnames(Cnames'First).CanonicalDomainName);
      IF Integer(Current_Byte) <
         DNS_Types.Message_Size-(12+DNS_Types.Header_Bits/8) then
         -- PTR to character of message
         Set_Unsigned_16(Output_Message.Bytes,Current_Byte+1,
            Unsigned_Types.Unsigned16(Qname_Location)+16#C000#);
         -- CNAME
         Output_message.Bytes(Current_Byte+3) := 16#00#;
         Output_message.Bytes(Current_Byte+4) := 16#05#;
         -- IN
         Output_message.Bytes(Current_Byte+5) := 16#00#;
         Output_message.Bytes(Current_Byte+6) := 16#01#;
         -- TTL
         Set_Unsigned_32(Output_message.Bytes,Current_Byte+7,Cnames(Cnames'First).TtlInSeconds);
         -- RDLENGTH
         Set_Unsigned_16(Output_message.Bytes,Current_Byte+11,Unsigned_Types.Unsigned16(Name_Length));
         Current_Byte := Current_Byte + 12;
      end if;
      Qname_Location := DNS_Types.QNAME_PTR_RANGE(
         (Integer(Current_Byte)) + DNS_Types.Header_Bits/8);
      I := 1;
      while I<=Name_Length and I<RR_Type.WireStringTypeIndex'Last and
            Integer(Current_Byte)+DNS_Types.Header_Bits/8<DNS_Types.Message_Size loop
         --# assert I<RR_Type.WireStringTypeIndex'Last and
         --#        I<=Name_Length and
         --#        Current_Byte >= Start_Byte and
         --#        Integer(Current_Byte)+DNS_Types.Header_Bits/8<DNS_Types.message_size and
         --#	    Answer_Count = Answer_Count~;
         Current_Byte := Current_Byte + 1;
         Output_message.Bytes(Current_Byte) := DNS_Types.Byte(Character'Pos(Cnames(Cnames'First).CanonicalDomainName(I)));
         I := I + 1;
      end loop;
      Output_Bytes := DNS_Types.message_length_range(Current_Byte + DNS_Types.message_bytes_range(DNS_Types.Header_Bits/8));
      --could prove a tighter bound than this, but who cares
      If Answer_Count < dns_types.Unsigned_Short'last then
         Answer_Count := Answer_Count + 1;
      ELSE
         Answer_Count := Dns_Types.Unsigned_Short'Last;
      END IF;
      IF Output_Message.Header.ANCount < dns_types.Unsigned_Short'last then
         Output_Message.Header.ANCount := Output_Message.Header.ANCount + 1;
      ELSE
         Output_Message.Header.ANCount := Dns_Types.Unsigned_Short'Last;
      END IF;
   END Process_Response_Cname;

