--WITH Ada.Text_IO;
package body Protected_Buffer_Pkg is

   ---------------------------
   -- Protected_Buffer_Type --
   ---------------------------

   protected body Protected_Buffer_Type is
      ---------------------
      -- numMsgsInBuffer --
      ---------------------
      function NumMsgsInBuffer return buffer_pkg.BufferIndexWithZeroType
      --# global in NumMsgs;
      --# return NumMsgs;
      is
      begin
         return NumMsgs;
      END NumMsgsInBuffer;

      --GRRR.  SPARK says this has to be a procedure because of
      --the (possible) I/O
      PROCEDURE CheckBufferFull(answer : out boolean)
      --# global in out protected_spark_io_05.spark_io_po;
      --#    in NumMsgs, Name, Debug;
      --# derives Answer from NumMsgs &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from NumMsgs, Name, Debug, Protected_SPARK_IO_05.SPARK_IO_PO;
      IS
      BEGIN
         IF NumMsgs = Buffer_Pkg.BufferSize THEN
           IF Debug THEN
               Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                  Name, 0);
               Protected_SPARK_IO_05.SPARK_IO_PO.Put_Line(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                  " full -- try increasing buffer_pkg.BufferSize",0);
            END IF;
            answer := True;
         ELSE
            answer := False;
         END IF;
      end CheckBufferFull;

      FUNCTION Expired(Slot : IN Buffer_Pkg.BufferIndexType) RETURN Boolean
      --# global in buffer, Ada.Real_Time.Clocktime;
      IS
         Now : Ada.Real_Time.Time;
         answer : boolean;
      BEGIN
         Now := Ada.Real_Time.Clock;
         answer := (Buffer(Slot).TimeStamp + dns_network.QUERY_TIME_LIMIT_SEC < Now);
         return answer;
      END Expired;

      ---------------
      -- MsgAtSlot --
      ---------------
      FUNCTION EntryAtSlot(S: IN Buffer_Pkg.BufferIndexType) RETURN
        buffer_pkg.BufferEntryType
      --# global in buffer;
      IS
      BEGIN
         RETURN Buffer(S);
      END EntryAtSlot;

      --Just does a simple linear search, can make this smarter if it affects performance
      FUNCTION FindSlotOfQueryWithConnectingId(ID: IN Dns_Types.Unsigned_Short) RETURN
         Buffer_Pkg.BufferIndexWithZeroType
      --# global in buffer;
      IS
      RetVal : Buffer_Pkg.BufferIndexWithZeroType;
      BEGIN
         RetVal := 0;
         FOR I IN Buffer_Pkg.BufferIndexType LOOP
            --# assert true;
            IF Buffer(I).Status = buffer_pkg.OCCUPIED and then Buffer(I).ConnectingID = ID then
               RetVal := I;
               EXIT;
            END IF;
         END LOOP;
         return RetVal;
      END FindSlotOfQueryWithConnectingId;

      ---------
      -- Add --
      ---------
      PROCEDURE Add(Msg: IN Dns_Types.dns_message;
         MsgLengthInBytes : in DNS_Types.message_Length_Range;
         Requester : IN Dns_Network.Network_Address_And_Port;
         PortNumber : IN Integer;
         WhereAdded : out buffer_pkg.BufferIndexType;
         Failure : out boolean)
      --# global in out buffer, NumMsgs, BufferNotEmpty, protected_spark_io_05.spark_io_po;
      --#    in Name, Debug, Ada.Real_Time.ClockTime;
      --# derives buffer from buffer, NumMsgs, Msg, MsgLengthInBytes,
      --#    Requester,PortNumber, Ada.Real_Time.ClockTime &
      --#    NumMsgs from buffer, NumMsgs &
      --#    BufferNotEmpty from buffer, NumMsgs, BufferNotEmpty &
      --#    protected_spark_io_05.spark_io_po from buffer, NumMsgs,
      --#       protected_spark_io_05.spark_io_po, Name, Debug,
      --#       Msg, MsgLengthInBytes, Requester, PortNumber, Ada.Real_Time.ClockTime &
      --#    WhereAdded from buffer, NumMsgs &
      --#    Failure from buffer, NumMsgs;
      IS
      BEGIN
         Failure := True;   --OUT parameters must always be assigned something
         WhereAdded := 1;
         if NumMsgsInBuffer < buffer_pkg.BufferSize then
            FOR I IN Buffer_Pkg.BufferIndexType LOOP
               IF Buffer(I).Status = buffer_pkg.FREE THEN
                  Buffer(I).Msg := Msg;
                  Buffer(I).NumBytes := MsgLengthInBytes;
                  Buffer(I).Status := Buffer_Pkg.OCCUPIED;
                  Buffer(I).Requester := Requester;
                  --this will get overwritten later if this is a query
                  Buffer(I).PortNumber := PortNumber;
                  WhereAdded := I;
                  Buffer(I).ConnectingID := 0;  --this will be set later
                  Buffer(I).TimeStamp := Ada.Real_Time.Clock;
                  NumMsgs := NumMsgs + 1;
                  BufferNotEmpty := true;
                  Failure := False;
                  exit;
               END IF;
               --# assert NumMsgs < buffer_pkg.BufferSize;
            END LOOP;
            IF debug then
              Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                  Name, 0);
              Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                  ":  Added msg ",0);
              Dns_Types.Print_Header_Info(Msg.Header);
              Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output," from server ",0);
               Dns_Network.Print_Server_Info(Buffer(WhereAdded).Requester);
              Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                 Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                 " ",0);
              Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                 Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                 " at slot ",0);
              Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
                 Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                 Integer(WhereAdded),2,10);
              Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
              Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                 Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                  " Port number initialized to ",0);
              Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
                 Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                  Buffer(WhereAdded).PortNumber,2,10);
              Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
                  Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
            end if;
         ELSE -- buffer full, can add warning code here if needed
            NULL;
         end if;
      end Add;

      ------------
      -- Delete --
      ------------
      PROCEDURE Delete(Slot: In Buffer_Pkg.BufferIndexType;
         Failure : OUT Boolean)
      --# global in out buffer, Protected_SPARK_IO_05.SPARK_IO_PO, numMsgs,
      --#    BufferNotEmpty; in Name, Debug;
      --# derives buffer from buffer, nummsgs, slot &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from buffer,
      --#       Protected_SPARK_IO_05.SPARK_IO_PO, numMsgs, Name, Debug, slot &
      --#    numMsgs from buffer, numMsgs, slot &
      --#    BufferNotEmpty from buffer, numMsgs, BufferNotEmpty, slot &
      --#    Failure from buffer, numMsgs, slot;
      is
      begin
         Failure := false;
         if numMsgsInBuffer > 0 then
            IF Buffer(Slot).Status = Buffer_Pkg.FREE THEN
               Failure := True;  --should never happen
            ELSE
               numMsgs := numMsgs - 1;
               if numMsgs = 0 then
                  BufferNotEmpty := false;
               END IF;
               if debug then
                  Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                     Name, 0);
                  Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                     ":  Deleted msg ",0);
                  Dns_Types.Print_Header_Info(Buffer(Slot).Msg.Header);
                  Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                     " from server ",0);
                  Dns_Network.Print_Server_Info(Buffer(Slot).Requester);
                  Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                     " at slot ",0);
                  Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                     Integer(Slot),3,10);
                  Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
                        Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
               END IF;
               --this sets Buffer(Slot).Status to FREE, resets time stamp
               Buffer(Slot) := Buffer_Pkg.EMPTY_BUFFER_ENTRY;
            END IF;
         ELSE --buffer is empty, deletion attempt should never happen but report if it does
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               "Error, attempt to delete from empty buffer, ignored",0);
            Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         end if;
      END Delete;

      PROCEDURE SetName(BufferName : IN BufferNameType)
      --# global out Name;
      --# derives Name from BufferName;
      is
      BEGIN
         Name := BufferName;
      END SetName;

      PROCEDURE setDebugging(Value : boolean)
      --# global out Debug;
      --# derives Debug from Value;
      is
      BEGIN
         Debug := Value;
      END SetDebugging;

      FUNCTION Debugging return boolean
      --# global in Debug;
      --# return Debug;
      IS
      BEGIN
         RETURN Debug;
      END Debugging;


      PROCEDURE SetConnectingID(Slot : IN Buffer_Pkg.BufferIndexType;
         ID : IN Dns_Types.Unsigned_Short)
      --# global in out Buffer;
      --# derives Buffer from Buffer, Slot, ID;
      IS
      BEGIN
         Buffer(Slot).ConnectingID := ID;
      END SetConnectingID;

      PROCEDURE SetPortNumber(Slot : IN Buffer_Pkg.BufferIndexType;
         Port : IN Integer)
      --# global in out Buffer, Protected_Spark_IO_05.SPARK_IO_PO;
      --#    in Name, Debug;
      --# derives Buffer from Buffer, Slot, Port &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from Protected_SPARK_IO_05.SPARK_IO_PO,
      --#       Name, Debug, Slot, Port;
      IS
      BEGIN
         Buffer(Slot).PortNumber := Port;
         IF Debugging THEN
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               Name, 0);
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               " set port number at slot ",0);
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               Integer(Slot),3,10);
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               " to ",0);
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               Port,3,10);
            Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         END IF;
      END SetPortNumber;

      PROCEDURE SetOriginalCaseDomainName(Slot : IN Buffer_Pkg.BufferIndexType;
         DomainName : IN RR_Type.WireStringType)
      --# global in out Buffer;
      --# derives Buffer from Buffer, Slot, DomainName;
      IS
      BEGIN
         Buffer(Slot).OriginalCaseDomainName := DomainName;
      END SetOriginalCaseDomainName;

      PROCEDURE SetRandomizedCaseDomainName(Slot : IN Buffer_Pkg.BufferIndexType;
         DomainName : IN RR_Type.WireStringType)
      --# global in out Buffer;
      --# derives Buffer from Buffer, Slot, DomainName;
      IS
      BEGIN
         Buffer(Slot).RandomizedCaseDomainName := DomainName;
      END SetRandomizedCaseDomainName;

      --Returns the next occupied slot found by the round robin pointer.
      --Blocks if buffer is empty
      ENTRY NextOccupiedSlot(Slot: OUT Buffer_Pkg.BufferIndexType)
         when BufferNotEmpty
      --# global in Buffer; in out RoundRobinPtr;
      --# derives Slot from Buffer, RoundRobinPtr &
      --#    RoundRobinPtr from RoundRobinPtr, Buffer;
      IS
         ctr : unsigned_types.Unsigned32;
         --Increments round robin pointer, wraps around
         PROCEDURE IncRR
         --# global in out RoundRobinPtr;
         --# derives RoundRobinPtr from RoundRobinPtr;
         IS
         BEGIN
            IF RoundRobinPtr = buffer_pkg.BufferSize THEN
               RoundRobinPtr := 1;
            ELSE
               RoundRobinPtr := RoundRobinPtr + 1;
            end if;
         END IncRR;

      BEGIN
         IncRR;
         Ctr := 1;
         Slot := RoundRobinPtr;
         WHILE Buffer(RoundRobinPtr).Status = Buffer_Pkg.FREE AND
            ctr <= buffer_pkg.BufferSize LOOP
            --# assert ctr <= buffer_pkg.BufferSize;
            IncRR;
            ctr := ctr + 1;
         END LOOP;
         if Buffer(RoundRobinPtr).Status = Buffer_Pkg.OCCUPIED then
            Slot := RoundRobinPtr;
         END IF;
      END NextOccupiedSlot;

      PROCEDURE PurgeExpiredQueries
      --# global in Ada.Real_Time.ClockTime, Name, Debug;
      --#    in out Buffer, Protected_SPARK_IO_05.SPARK_IO_PO, NumMsgs, BufferNotEmpty;
      --# derives Buffer from Buffer, Ada.Real_Time.ClockTime, NumMsgs &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from
      --#       Protected_SPARK_IO_05.SPARK_IO_PO, Name, Buffer, Debug, NumMsgs,
      --#       Ada.Real_Time.ClockTime &
      --#    NumMsgs from NumMsgs, Ada.Real_Time.ClockTime, Buffer &
      --#    BufferNotEmpty from Buffer, BufferNotEmpty, NumMsgs,
      --#       Ada.Real_Time.ClockTime;
      IS
         ExpiredQuery : Boolean;
         Failure : Boolean;
         begin
            FOR Slot IN Buffer_Pkg.BufferIndexType LOOP
               --# assert true;
               ExpiredQuery := Expired(Slot);
               IF ExpiredQuery THEN
                  Delete(Slot, Failure);
                  --Failure ignored here, make flow error go away
                  IF Debugging and Failure = Failure THEN
                     Protected_SPARK_IO_05.SPARK_IO_PO.Put_Line(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                     " (query expired) ",0);
                  END IF;
               END IF;
            END LOOP;
         END PurgeExpiredQueries;

      -----------------
      -- PrintBuffer --
      -----------------

      PROCEDURE PrintBuffer
      --# global in Buffer,Name,numMsgs; in out Protected_SPARK_IO_05.SPARK_IO_PO;
      --# derives Protected_SPARK_IO_05.SPARK_IO_PO from buffer, Name, numMsgs,
      --#    protected_SPARK_IO_05.SPARK_IO_PO;
      is
      BEGIN
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            "CONTENTS OF ", 0);
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            Name, 0);
         Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         FOR I IN buffer_pkg.BufferIndexType LOOP
            --# assert true;
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               Integer(I),3,10);
            Dns_Types.Print_Header_Info((Buffer(I).Msg.Header));
            Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               "   ", 0);
            dns_network.Print_server_Info(Buffer(I).Requester);
            --no enumeration IO in SPARK, sigh
            CASE Buffer(I).Status IS
               WHEN Buffer_Pkg.FREE =>
                  Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                     "   FREE ", 0);
               WHEN Buffer_Pkg.OCCUPIED =>
                  Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
                     Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
                     "   OCCUPIED ", 0);
            end case;
            Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
               Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
         END LOOP;
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_String(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
               "Number of msgs in buffer = ",0);
         Protected_SPARK_IO_05.SPARK_IO_PO.Put_Integer(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,
            Integer(numMsgs),3,10);
         Protected_SPARK_IO_05.SPARK_IO_PO.New_Line(
            Protected_SPARK_IO_05.SPARK_IO_PO.Standard_Output,1);
      END PrintBuffer;
   end Protected_Buffer_Type;

end Protected_Buffer_Pkg;
