WITH Buffer_Pkg;
USE TYPE Buffer_Pkg.Occupied_Status_Type;
with rr_type;
WITH Dns_Types;
WITH Dns_Network;
with unsigned_types;
WITH Protected_SPARK_IO_05;
with ada.Real_Time;
USE TYPE Dns_Types.Unsigned_Short;
USE TYPE Unsigned_Types.Unsigned32;
use type Ada.Real_Time.Time;
--# inherit buffer_pkg, rr_type, dns_types, dns_network, unsigned_types,
--# ada.real_time, protected_spark_io_05;

PACKAGE Protected_Buffer_Pkg IS
   MaxBufferNameLength : constant Natural := 16;
   subtype BufferNameIndexType is Integer range 1..MaxBufferNameLength;
   subtype BufferNameType is String(BufferNameIndexType);
   EMPTY_BUFFER_NAME : CONSTANT BufferNameType := "                ";
   PROTECTED TYPE Protected_Buffer_Type IS
      pragma priority(0);

      --SPARK/Ravenscar says that only one of these methods can be an entry
      FUNCTION EntryAtSlot(S: IN Buffer_Pkg.BufferIndexType) RETURN
        buffer_pkg.BufferEntryType;
      --# global in protected_buffer_type;

      FUNCTION NumMsgsInBuffer RETURN buffer_pkg.BufferIndexWithZeroType;
      --# global in protected_buffer_type;

      FUNCTION Expired(Slot : IN Buffer_Pkg.BufferIndexType) RETURN Boolean;
      --# global in protected_buffer_type, ada.Real_Time.clocktime;

      Procedure CheckBufferFull(Answer : OUT boolean);
      --# global in out Protected_SPARK_IO_05.SPARK_IO_PO;
      --#    in protected_buffer_type;
      --# derives Answer from protected_buffer_type &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from protected_buffer_type,
      --#    Protected_SPARK_IO_05.SPARK_IO_PO;

      FUNCTION FindSlotOfQueryWithConnectingId(ID: IN Dns_Types.Unsigned_Short) RETURN
         Buffer_Pkg.BufferIndexWithZeroType;
      --# global in protected_buffer_type;

      PROCEDURE Add(Msg: IN Dns_Types.DNS_Message;
         MsgLengthInBytes : IN DNS_Types.Message_Length_Range;
         Requester : IN Dns_Network.Network_Address_And_Port;
         PortNumber : IN Integer;
         WhereAdded : OUT Buffer_Pkg.BufferIndexType;
         Failure : out boolean);
      --# global in out protected_buffer_type, Protected_SPARK_IO_05.SPARK_IO_PO;
      --#   in Ada.Real_Time.ClockTime;
      --# derives protected_buffer_type from protected_buffer_type, Msg,
      --#    MsgLengthInBytes, Requester, PortNumber, Ada.Real_Time.ClockTime &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from Protected_SPARK_IO_05.SPARK_IO_PO,
      --#       protected_buffer_type, Msg, MsgLengthInBytes, Requester, PortNumber,
      --#       Ada.Real_Time.ClockTime &
      --#    WhereAdded from protected_buffer_type &
      --#    Failure from protected_buffer_type;
      --# pre Integer(MsgLengthInBytes) > DNS_Types.Header_Bits/8;

      PROCEDURE Delete(Slot: In Buffer_Pkg.BufferIndexType;
                       Failure : OUT Boolean);
      --# global in out protected_buffer_type, Protected_SPARK_IO_05.SPARK_IO_PO;
      --# derives protected_buffer_type from protected_buffer_type, slot &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from Protected_SPARK_IO_05.SPARK_IO_PO,
      --#       protected_buffer_type, slot &
      --#    Failure from protected_buffer_type, slot;

      ENTRY NextOccupiedSlot(Slot: out Buffer_Pkg.BufferIndexType);
      --# global in out protected_buffer_type;
      --# derives protected_buffer_type from protected_buffer_type &
      --#    slot from protected_buffer_type;

      PROCEDURE SetName(BufferName : IN BufferNameType);
      --# global in out protected_buffer_type;
      --# derives protected_buffer_type from protected_buffer_type, BufferName;

      PROCEDURE SetConnectingID(Slot : IN Buffer_Pkg.BufferIndexType;
         ID : IN Dns_Types.Unsigned_Short);
      --# global in out protected_buffer_type;
      --# derives protected_buffer_type from protected_buffer_type, Slot, ID;

      PROCEDURE SetPortNumber(Slot : IN Buffer_Pkg.BufferIndexType;
         Port : IN Integer);
      --# global in out protected_buffer_type, Protected_SPARK_IO_05.SPARK_IO_PO;
      --# derives protected_buffer_type from protected_buffer_type, Slot, Port &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from Protected_SPARK_IO_05.SPARK_IO_PO,
      --#       protected_buffer_type, Slot, Port;

      PROCEDURE SetOriginalCaseDomainName(Slot : IN Buffer_Pkg.BufferIndexType;
         DomainName : IN RR_Type.WireStringType);
      --# global in out protected_buffer_type;
      --# derives protected_buffer_type from protected_buffer_type, Slot, DomainName;

      PROCEDURE SetRandomizedCaseDomainName(Slot : IN Buffer_Pkg.BufferIndexType;
         DomainName : IN RR_Type.WireStringType);
      --# global in out protected_buffer_type;
      --# derives protected_buffer_type from protected_buffer_type, Slot, DomainName;

      PROCEDURE PurgeExpiredQueries;
      --# global in Ada.Real_Time.ClockTime;
      --#    in out protected_buffer_type, Protected_SPARK_IO_05.SPARK_IO_PO;
      --# derives protected_buffer_type from protected_buffer_type, Ada.Real_Time.ClockTime &
      --#    Protected_SPARK_IO_05.SPARK_IO_PO from
      --#       Protected_SPARK_IO_05.SPARK_IO_PO, protected_buffer_type,
      --#       Ada.Real_Time.ClockTime;

      PROCEDURE PrintBuffer; --DEBUG
      --# global in protected_buffer_type; in out Protected_SPARK_IO_05.SPARK_IO_PO;
      --# derives Protected_SPARK_IO_05.SPARK_IO_PO from protected_buffer_type,
      --#    Protected_SPARK_IO_05.SPARK_IO_PO;

      PROCEDURE SetDebugging(Value : boolean);
      --# global in out protected_buffer_type;
      --# derives protected_buffer_type from protected_buffer_type, Value;

      FUNCTION Debugging RETURN Boolean;
      --# global in protected_buffer_type;

   PRIVATE
      --SPARK says these all have to be initialized here
      Buffer : Buffer_Pkg.BufferType :=
         Buffer_Pkg.BufferType'(others => buffer_pkg.EMPTY_BUFFER_ENTRY);
      NumMsgs : buffer_pkg.BufferIndexWithZeroType := 0;
      RoundRobinPtr : Buffer_Pkg.BufferIndexType :=
         buffer_pkg.BufferIndexType'First;
      Name : BufferNameType := EMPTY_BUFFER_NAME;
      BufferNotEmpty : Boolean := False;
      Debug : Boolean := False;
   END Protected_Buffer_Type;
END Protected_Buffer_Pkg;

