SEPARATE(Wire_To_Records_Pkg)
   --had to make this a separate procedure so that the Simplifier could handle it
PROCEDURE Build_And_Insert_Record (
      Response_Message : IN OUT    Dns_Types.DNS_Message;
      --the response we got from the wire
      Response_Bytes : IN   DNS_Types.Message_Length_Range;
      Record_Type    : IN     DNS_Types.Query_Type;
      --the type of record to be built/added
      BytePtr : IN     Dns_Types.Message_Bytes_Range;
      --first byte of RDATA section
      Record_Class : IN     DNS_Types.Query_Class;
      TTL          : IN     Unsigned_Types.Unsigned32;
      RDLength     : IN     Unsigned_Types.Unsigned16;
      Domainname   : IN     RR_Type.WireStringType;
      ValidRecord  : IN OUT Boolean) IS
   --always true upon entry, but might change

   ARecord        : Rr_Type.A_Record_Type.ARecordType;
   AAAARecord     : Rr_Type.AAAA_Record_Type.AAAARecordType;
   CNAMERecord    : Rr_Type.CNAME_Record_Type.CNAMERecordType;
   MXRecord       : Rr_Type.MX_Record_Type.MXRecordType;
   NSRecord       : Rr_Type.NS_Record_Type.NSRecordType;
   NXDOMAINRecord : Rr_Type.NXDOMAIN_Record_Type.NXDOMAINRecordType;
   PTRRecord      : Rr_Type.PTR_Record_Type.PTRRecordType;
   SOARecord      : Rr_Type.SOA_Record_Type.SOARecordType;

   CorruptRecord : Rr_Type.Corrupt_Record_Type.CorruptRecordType;
   TempIp        : Unsigned_Types.Unsigned32;                     --for 0.0.0.0 check

   FirstLoc        : Integer                := 1;
   SecondLoc       : Integer;
   ThirdLoc        : Integer;
   ShortenedDomain : Rr_Type.WireStringType;
   PerDomain       : Rr_Type.WireStringType;

   --DNSSEC records
   DNSKeyRecord : Rr_Type.Dnskey_Record_Type.DNSKeyRecordType;
   RRSIGRecord  : Rr_Type.RRSIG_Record_Type.RRSIGRecordType;
   NSECRecord   : Rr_Type.NSEC_Record_Type.NSECRecordType;

   Success : Boolean;

BEGIN
   CASE Record_Type IS
      WHEN Dns_Types.A =>
         --A records must have at least 4 bytes in RDATA section
         IF BytePtr <= Dns_Types.Message_Bytes_Range'Last-3 THEN

            --Get Ip, check if 0.0.0.0
            TempIp := Get_Unsigned_32(Response_Message.Bytes, BytePtr);
            IF Flags.Remap THEN
               IF Response_Message.Bytes(Byteptr)/=10 THEN
                  Protected_Spark_Io_05.SPARK_IO_PO.Put_String(
                     Protected_Spark_Io_05.SPARK_IO_PO.Standard_Output,"Badness IP: ",0);
                  Protected_Spark_Io_05.SPARK_IO_PO.Put_Integer(
                     Protected_Spark_Io_05.SPARK_IO_PO.Standard_Output, Integer(Response_Message.Bytes(BytePtr)), 4, Base=>10);
                  Protected_Spark_Io_05.SPARK_IO_PO.Put_Integer(
                     Protected_Spark_Io_05.SPARK_IO_PO.Standard_Output, Integer(Response_Message.Bytes(BytePtr+1)), 4, Base=>10);
                  Protected_Spark_Io_05.SPARK_IO_PO.Put_Integer(
                     Protected_Spark_Io_05.SPARK_IO_PO.Standard_Output, Integer(Response_Message.Bytes(BytePtr+2)), 4, Base=>10);
                  Protected_Spark_Io_05.SPARK_IO_PO.Put_Integer(
                     Protected_Spark_Io_05.SPARK_IO_PO.Standard_Output, Integer(Response_Message.Bytes(BytePtr+3)), 4, Base=>10);
                  Protected_Spark_Io_05.SPARK_IO_PO.New_Line(
                     Protected_Spark_Io_05.SPARK_IO_PO.Standard_Output, 1);
                  Response_Message.Bytes(Byteptr) := 0;
                  Response_Message.Bytes(Byteptr+1) := 0;
                  Response_Message.Bytes(Byteptr+2) := 0;
                  Response_Message.Bytes(Byteptr+3) := 0;
                  TempIp := 0;
               ELSE
                  Response_Message.Bytes(Byteptr) := 10;
               end if;
            END IF;

            -- If all zeros
            IF(TempIp = 0) THEN
               ShortenedDomain := To_Lower(Domainname);
               PerDomain := ShortenedDomain;
               PerDomain(FirstLoc) := '.';

               ThirdLoc := (FirstLoc + Character'Pos(Domainname(FirstLoc))) + 1;
               SecondLoc := FirstLoc;
               FindEnd:
               while thirdloc<=domainname'last LOOP
                     --# assert FirstLoc>=1 and FirstLoc<=domainname'last and
                     --# byteptr <= dns_types.Message_Bytes_Range'last-3 and
                     --# thirdloc>=1 and thirdloc<=domainname'last;
                  SecondLoc := FirstLoc;
                  FirstLoc := ThirdLoc;
                  PerDomain(FirstLoc) := '.';
                  ThirdLoc := (FirstLoc + 1) + Character'Pos(ShortenedDomain(FirstLoc));
                  EXIT FindEnd WHEN ThirdLoc>Domainname'Last OR ELSE
                     Character'Pos(Domainname(ThirdLoc)) = 0;
               END LOOP FindEnd;

               FOR I IN Natural RANGE 1..Shorteneddomain'Last-Secondloc LOOP
                  --# assert secondloc = secondloc% and
                  --# byteptr <= dns_types.Message_Bytes_Range'last-3 and
                  --# secondloc>=1 and secondloc+i<=domainname'last;
                  Shorteneddomain(I) := Shorteneddomain(Secondloc+I);
               END LOOP;
               --shortenedDomain(1..shortenedDomain'Last-secondLoc) := shortenedDomain(secondLoc+1..shortenedDomain'Last);
               ShortenedDomain := To_Lower(ShortenedDomain);
               Protected_Spark_Io_05.SPARK_IO_PO.Put_String(
                  Protected_Spark_Io_05.SPARK_IO_PO.Standard_Output,"Blocking: ",0);
               Protected_Spark_Io_05.SPARK_IO_PO.Put_Line(
                  Protected_Spark_Io_05.SPARK_IO_PO.Standard_Output,ShortenedDomain,0);
               CorruptRecord := Build_Corrupt_Record(Response_Message,BytePtr,TTL,Record_Class);
               Dns_Table_Pkg.DNS_Table.InsertCorruptRecord(ShortenedDomain, CorruptRecord);
            END IF;
            ARecord := Build_A_Record(Response_Message,BytePtr,TTL,Record_Class);
            Dns_Table_Pkg.Dns_Table.InsertARecord(DomainName, ARecord);
         ELSE
            ValidRecord := False;
         END IF;
      WHEN Dns_Types.AAAA =>
         --AAAA records must have at least 16 bytes in RDATA section
         IF BytePtr <= Dns_Types.Message_Bytes_Range'Last-15 THEN
            AAAARecord := Build_AAAA_Record(Response_Message,BytePtr,TTL,Record_Class);
            Dns_Table_Pkg.Dns_Table.InsertAAAARecord(DomainName, AAAARecord);
         ELSE
            ValidRecord := False;
         END IF;
      WHEN Dns_Types.CNAME =>
         --RDLENGTH can't be more than the maximum length of a wire name
         IF RDLength > Unsigned_Types.Unsigned16(Rr_Type.WireStringTypeIndex'Last) THEN
            ValidRecord := False;
            --CNAME records must have at least RDLENGTH bytes in RDATA section
         ELSIF BytePtr <= (Dns_Types.Message_Bytes_Range'Last-Dns_Types.Message_Bytes_Range(RDLength))+1 THEN
            Build_CNAME_Record(Response_Message,BytePtr,TTL,Record_Class,CNAMERecord,Success);
            IF Success THEN
               Dns_Table_Pkg.Dns_Table.InsertCNAMERecord(DomainName, CNAMERecord);
            END IF;
         ELSE
            ValidRecord := False;
         END IF;

      WHEN Dns_Types.MX =>
         --MX records must have at least 6 bytes in RDATA section
         IF BytePtr <= Dns_Types.Message_Bytes_Range'Last-5 AND Response_Bytes >=DNS_Types.Header_Bits/8+5 THEN
            MXRecord := Build_MX_Record(Response_Message,Response_Bytes,BytePtr,TTL,Record_Class);
            Dns_Table_Pkg.Dns_Table.InsertMXRecord(DomainName, MXRecord);
         ELSE
            ValidRecord := False;
         END IF;

      WHEN Dns_Types.NS =>  --just like CNAME
         --RDLENGTH can't be more than the maximum length of a wire name
         IF RDLength > Unsigned_Types.Unsigned16(Rr_Type.WireStringTypeIndex'Last) THEN
            ValidRecord := False;
            --NS records must have at least RDLENGTH bytes in RDATA section
         ELSIF BytePtr <= (Dns_Types.Message_Bytes_Range'Last-Dns_Types.Message_Bytes_Range(RDLength))+1 THEN
            Build_NS_Record(Response_Message, BytePtr, TTL, Record_Class, NSRecord, Success);
            IF Success THEN
               Dns_Table_Pkg.Dns_Table.InsertNSRecord(DomainName, NSRecord);
            ELSE
               ValidRecord := False;
            END IF;
         ELSE
            ValidRecord := False;
         END IF;

      WHEN Dns_Types.NXDOMAIN =>
         NXDOMAINRecord := Build_NXDOMAIN_Record(TTL,Record_Class);
         Dns_Table_Pkg.Dns_Table.InsertNXDOMAINRecord(DomainName, NXDOMAINRecord);

      WHEN Dns_Types.PTR =>  --just like CNAME
         --RDLENGTH can't be more than the maximum length of a wire name
         IF RDLength > Unsigned_Types.Unsigned16(Rr_Type.WireStringTypeIndex'Last) THEN
            ValidRecord := False;
            --PTR records must have at least RDLENGTH bytes in RDATA section
         ELSIF BytePtr <= (Dns_Types.Message_Bytes_Range'Last-Dns_Types.Message_Bytes_Range(RDLength))+1 THEN
            Build_PTR_Record(Response_Message, BytePtr, TTL, Record_Class, PTRRecord, Success);
            IF SUCCESS THEN
               Dns_Table_Pkg.Dns_Table.InsertPTRRecord(DomainName, PTRRecord);
            ELSE
               ValidRecord := False;
            END IF;
         ELSE
            ValidRecord := False;
         END IF;

      WHEN Dns_Types.SOA =>
         --RDLENGTH can't be more than twice maximum length of a wire name plus fixed length
         --data size (20 bytes)
         IF RDLength > 2*Unsigned_Types.Unsigned16(Rr_Type.WireStringTypeIndex'Last) + 20 THEN
            ValidRecord := False;
            --SOA records must have at least 20 bytes in RDATA section
         ELSIF BytePtr <= (Dns_Types.Message_Bytes_Range'Last-20)+1 THEN
            Build_SOA_Record(Response_Message, BytePtr, TTL, Record_Class, SOARecord, Success);
            IF SUCCESS THEN
               Dns_Table_Pkg.Dns_Table.InsertSOARecord(DomainName, SOARecord);
            ELSE
               ValidRecord := False;
            END IF;
         ELSE
            ValidRecord := False;
         END IF;

      --DNSSEC records
      WHEN Dns_Types.DNSKEY =>
         --RDLENGTH must be at least 4 (really at least the smallest key length, but who cares)
         --RDLENGTH can't be more than 4 + the maximum length of a DNS key
         IF RDLength < 4 OR RDLength > 4 + Unsigned_Types.Unsigned16(Rr_Type.Dnskey_Record_Type.MaxDNSKeyLength) THEN
            ValidRecord := False;
            --DNSKEY records must have at least one byte after the first 4 bytes in the RDATA section
            --(there's a tighter bound, but who cares)
         ELSIF BytePtr + 4 > Dns_Types.Message_Bytes_Range'Last THEN
            ValidRecord := False;
         ELSE
            Build_DNSKEY_Record(Response_Message,BytePtr,RDLength,TTL,Record_Class,DNSKEYRecord,Success);
            IF Success THEN
               Dns_Table_Pkg.Dns_Table.InsertDNSKEYRecord(DomainName, DNSKEYRecord);
            END IF;
         END IF;
      WHEN Dns_Types.NSEC =>
         --RDLength has to be at least 2 to meet precondition
         IF RDLength < 2 OR RDlength > Unsigned_Types.Unsigned16(Dns_Types.Message_Bytes_Range'Last) THEN
            ValidRecord := False;
         ELSE
            Build_NSEC_Record(Response_Message,BytePtr,RDLength,TTL,Record_Class,NSECRecord,Success);
            IF Success THEN
               Dns_Table_Pkg.Dns_Table.InsertNSECRecord(DomainName, NSECRecord);
            END IF;
         END IF;
      WHEN Dns_Types.RRSIG =>
         --RDLENGTH can't be more than 18 + the maximum length of a domainname +
         --the maximum length of a DNS key
         --RDLENGTH must be at least 18 (there's actually a tighter bound, but who cares)
         IF RDLength < 18 OR RDLength > 18 + Unsigned_Types.Unsigned16(Rr_Type.MaxDomainNameLength)
               + Unsigned_Types.Unsigned16(Rr_Type.Dnskey_Record_Type.MaxDNSKeyLength) THEN
            ValidRecord := False;
            --RRSIG records must have at least one byte after the first 18 bytes in the RDATA section
            --(there's a tighter bound, but who cares)
         ELSIF BytePtr + 18 > Dns_Types.Message_Bytes_Range'Last THEN
            ValidRecord := False;
         ELSE
            Build_RRSIG_Record(Response_Message,BytePtr,RDLength,TTL,Record_Class,RRSIGRecord,Success);
            IF Success THEN
               Dns_Table_Pkg.Dns_Table.InsertRRSIGRecord(DomainName, RRSIGRecord);
            END IF;
         END IF;

      WHEN OTHERS =>
         NULL;  --Should say something here
   END CASE;
END Build_And_Insert_Record;


