SEPARATE(Wire_To_Records_Pkg)
   PROCEDURE Build_DNSKEY_Record(Packet : IN Dns_Types.DNS_Message;
      RDATAStartByte : IN Dns_Types.message_bytes_range;
      RDLength: IN Unsigned_Types.Unsigned16;  --need this to determine how long the key is
      TTL : IN Unsigned_Types.Unsigned32;
      Class : IN Dns_Types.Query_Class;
      ReturnedRec : OUT Rr_Type.DNSKEY_Record_Type.DNSKEYRecordType;
      Success : OUT Boolean)
      IS
         Now : Ada.Real_Time.Time;
         BytePtr : Dns_Types.message_bytes_range;
         bytesRead: Unsigned_Types.Unsigned16;
         Index : Rr_Type.Dnskey_Record_Type.DNSKeyStringTypeIndex;
         SrcByte0 : Dns_Types.Byte;
         SrcByte1 : Dns_Types.Byte;
         SrcByte2 : dns_types.Byte;
      BEGIN
         Success := True;
         ReturnedRec := rr_type.dnskey_record_type.blankDNSKEYRecord; --pacify the SPARK monster
   	     ReturnedRec.TTLInSeconds := TTL;
         CASE Class IS
            when dns_types.IN_CLASS =>
               ReturnedRec.Class := Rr_Type.INTERNET;
            --sun will go nova before these get executed
      	    WHEN Dns_Types.CH_CLASS =>
               ReturnedRec.Class := Rr_Type.CH;
            WHEN Dns_Types.HS_CLASS =>
               ReturnedRec.Class := Rr_Type.HS;
            --SPARK ensures this never happens due to precondition
            WHEN Dns_Types.NONE_CLASS | Dns_Types.ANY_CLASS =>
               ReturnedRec.Class := Rr_Type.INTERNET;
         END CASE;
         BytePtr := RDATAStartByte;
         --FLAGS (2 bytes)
   	     ReturnedRec.flags := Get_Unsigned_16(Packet.Bytes, BytePtr);
         BytePtr := BytePtr + 2;

         --PROTOCOL (1 byte)
         ReturnedRec.Protocol := unsigned_types.Unsigned8(Packet.Bytes(BytePtr));
         BytePtr := BytePtr + 1;

         --ALGORITHM (1 byte)
         ReturnedRec.Algorithm:= unsigned_types.Unsigned8(Packet.Bytes(BytePtr));
         BytePtr := BytePtr + 1;

         --PUBLIC KEY (RDLENGTH-4 bytes on the wire)
         bytesRead:= 0;
         Index := Rr_Type.Dnskey_Record_Type.DNSKeyStringTypeIndex'First;

         --need to base64 encode what's on the wire
         WHILE BytesRead <= (RDLENGTH-4)-3 AND BytePtr + 2 < Dns_Types.message_bytes_range'Last AND
         Index + 3 < Rr_Type.Dnskey_Record_Type.DNSKeyStringTypeIndex'Last loop
            --# assert BytesRead <= (RDLENGTH-4)-3 AND BytePtr + 2 < Dns_Types.message_bytes_range'Last
            --# AND Index >= Rr_Type.Dnskey_Record_Type.DNSKeyStringTypeIndex'First
            --# and Index + 3 < Rr_Type.Dnskey_Record_Type.DNSKeyStringTypeIndex'Last;

            --three bytes on wire
            SrcByte0 := Packet.Bytes(BytePtr);   --aaaaaabb
            SrcByte1 := Packet.Bytes(BytePtr+1); --bbbbcccc
            SrcByte2 := Packet.Bytes(BytePtr+2); --ccdddddd
            --four bytes in record
            ReturnedRec.key(Index) := Base64ValToChar(SrcByte0/4); --aaaaaa
            ReturnedRec.Key(Index+1) := Base64ValToChar((SrcByte0 mod 4)*16 + SrcByte1/16); --bbbbbb
            ReturnedRec.Key(Index+2) := Base64ValToChar((SrcByte1 mod 16)*4 + SrcByte2/64); --cccccc
            ReturnedRec.Key(Index+3) := Base64ValToChar(SrcByte2 mod 64); --dddddd

            bytesRead:= bytesRead+3;
            BytePtr := BytePtr+3;
            Index := Index+4;
         END LOOP;

         --if number of bytes not divisible by 3, must handle as
         --described in http://en.wikipedia.org/wiki/Base64
      	 if Index+3 <= Rr_Type.Dnskey_Record_Type.DNSKeyStringTypeIndex'Last and
      	 BytePtr + 1 <= DNS_Types.message_bytes_range'Last then
         	 CASE (bytesRead-(RDLENGTH-4)) is
               WHEN 0 => null;
               WHEN 1 =>
                  SrcByte0 := Packet.Bytes(BytePtr);
                  SrcByte1 := 0;
                  ReturnedRec.key(Index) := Base64ValToChar(SrcByte0/4); --aaaaaa
                  ReturnedRec.Key(Index+1) := Base64ValToChar((SrcByte0 mod 4)*16 + SrcByte1/16); --bbbbbb
                  ReturnedRec.Key(Index+2) := '=';
                  ReturnedRec.Key(Index+3) := '=';
                  BytesRead := BytesRead+1;
               WHEN 2 => --some redundancy with above, but easier to read
                  SrcByte0 := Packet.Bytes(BytePtr);
                  SrcByte1 := Packet.Bytes(BytePtr+1);
                  SrcByte2 := 0;
                  ReturnedRec.key(Index) := Base64ValToChar(SrcByte0/4); --aaaaaa
                  ReturnedRec.Key(Index+1) := Base64ValToChar((SrcByte0 mod 4)*16 + SrcByte1/16); --bbbbbb
                  ReturnedRec.Key(Index+2) := Base64ValToChar((SrcByte1 mod 16)*4 + SrcByte2/64); --cccccc
                  ReturnedRec.Key(Index+3) := '=';
                  BytesRead := BytesRead+2;
               WHEN OTHERS => NULL;
            END CASE;
            ReturnedRec.KeyLength := Index+3;
         END IF;  --Index and BytePtr in bounds

         Now := Ada.Real_Time.Clock;
         ReturnedRec.TimeStamp := Now;

         --testing for validity here is slightly less efficient, but simplifies analysis
         --could validate the FLAGS field here at some point (only valid values are
         -- 0, 256 and 257)
         IF ReturnedRec.Protocol /= Rr_Type.Dnskey_Record_Type.ONLY_SUPPORTED_PROTOCOL OR
            bytesRead /= RDLength-4 THEN
            Success := False;
         END IF;
   end Build_DNSKEY_Record;

