SEPARATE(Wire_To_Records_Pkg)
   PROCEDURE Build_NSEC_Record(Packet : IN Dns_Types.DNS_Message;
      RDATAStartByte : IN Dns_Types.Message_Bytes_Range;
      RDLength: IN Unsigned_Types.Unsigned16;  --need this to determine how long the key is
      TTL : IN Unsigned_Types.Unsigned32;
      Class : IN Dns_Types.Query_Class;
      ReturnedRec : OUT Rr_Type.NSEC_Record_Type.NSECRecordType;
      Success : OUT Boolean)
   IS
      BytePtr : Dns_Types.Message_Bytes_Range;
      NameStartPtr : Dns_Types.Message_Bytes_Range;
      CurrentBlock : Rr_Type.Nsec_Record_Type.BlockNumberValue;
      ValidMap : Boolean;
      CurrentLength : Rr_Type.Nsec_Record_Type.BlockLengthValue;
      Now : Ada.Real_Time.Time;

      BEGIN
         ValidMap := True;
         ReturnedRec := rr_type.nsec_record_type.blankNSECRecord; --pacify the SPARK monster
         ReturnedRec.TTLInSeconds := TTL;
         CASE Class IS
            when dns_types.IN_CLASS =>
               ReturnedRec.Class := Rr_Type.INTERNET;
            --sun will go nova before these get executed
      	    WHEN Dns_Types.CH_CLASS =>
               ReturnedRec.Class := Rr_Type.CH;
            WHEN Dns_Types.HS_CLASS =>
               ReturnedRec.Class := Rr_Type.HS;
            --SPARK ensures this never happens due to precondition
            WHEN Dns_Types.NONE_CLASS | Dns_Types.ANY_CLASS =>
               ReturnedRec.Class := Rr_Type.INTERNET;
         END CASE;

         --NAME
         BytePtr := RDATAStartByte;
         --copy next domain name from the wire to the record, expanding any pointers
         MakePtrFreeDomainNameFromWire(Packet, BytePtr, ReturnedRec.NextDomainName, Success);

         --BITMAP
         IF Success THEN
            NameStartPtr := BytePtr;
            FindFirstByteAfterName(Packet, NameStartPtr, BytePtr, Success);
         end if;
         IF Success THEN
            CurrentBlock := 0;  --needed for first path into loop only
      	    CurrentLength := 1; -- "
            WHILE BytePtr < RDATAStartByte + dns_types.Message_Bytes_Range(RDLength-1)
      	    AND ValidMap LOOP
               --# assert BytePtr >= dns_types.message_bytes_range'first
               --# and BytePtr < RDATAStartByte + dns_types.Message_Bytes_Range(RDLength-1)
               --# and RDLength >= 2 and RDLength <= unsigned_types.unsigned16(dns_types.message_bytes_range'last)
               --# and CurrentBlock >= rr_type.nsec_record_type.blockNumberValue'first
               --# and CurrentBlock <= rr_type.nsec_record_type.blockNumberValue'last
               --# and CurrentLength >= rr_type.Nsec_Record_Type.BlockLengthValue'first
               --# and CurrentLength <= rr_type.Nsec_Record_Type.BlockLengthValue'last
               --# and ValidMap;
               if BytePtr + 1 > dns_types.Message_Bytes_Range'last or else
               Packet.Bytes(BytePtr+1) < dns_types.byte(rr_type.Nsec_Record_Type.BlockLengthValue'first) or else
               Packet.Bytes(BytePtr+1) > dns_types.byte(rr_type.Nsec_Record_Type.BlockLengthValue'last) or else
               CurrentBlock = rr_type.Nsec_Record_Type.blockNumberValue'last then
                  ValidMap := false;
               else
                  --BytePtr now points to the first byte of current block of bitmap
                  CurrentBlock := CurrentBlock+1;
                  ReturnedRec.BlockNumbers(CurrentBlock) := Packet.Bytes(BytePtr);
                  CurrentLength := rr_type.Nsec_Record_Type.BlockLengthValue(Packet.Bytes(BytePtr+1));
                  ReturnedRec.BlockLengths(CurrentBlock) := CurrentLength;
                  -- check there's enough room for the current block
                  if BytePtr + (dns_types.Message_Bytes_Range(CurrentLength)+1) >
                  dns_types.Message_Bytes_Range'last then
                     ValidMap := false;
                  else
                     FOR ByteInBitmap IN Rr_Type.Nsec_Record_Type.BitmapIndex
                     RANGE Rr_Type.Nsec_Record_Type.BitmapIndex'First .. CurrentLength LOOP
                     --# assert CurrentLength = CurrentLength%
                     --# and BytePtr >= dns_types.message_bytes_range'first
                     --# and BytePtr < RDATAStartByte + dns_types.Message_Bytes_Range(RDLength-1)
                     --# and RDLength >= 2 and RDLength <= unsigned_types.unsigned16(dns_types.message_bytes_range'last)
                     --# and CurrentBLock >= rr_type.nsec_record_type.blockNumberArrayIndex'first
                     --# and CurrentBLock <= rr_type.nsec_record_type.blockNumberArrayIndex'last
                     --# and CurrentLength >= rr_type.Nsec_Record_Type.BlockLengthValue'first
                     --# and CurrentLength <= rr_type.Nsec_Record_Type.BlockLengthValue'last
                     --# and ByteInBitMap >= Rr_Type.Nsec_Record_Type.BitmapIndex'First
                     --# and ByteInBitMap <= CurrentLength
                     --# and BytePtr + (dns_types.Message_Bytes_Range(CurrentLength)+1) <=
                     --#    dns_types.Message_Bytes_Range'last
                     --# and ValidMap;
                        ReturnedRec.Bitmaps(CurrentBlock)(ByteInBitmap) :=
                           Packet.Bytes(BytePtr + (dns_types.Message_Bytes_Range(ByteInBitmap)+1));
                     end loop; --building current block of bitmap
                  end if;  --current block of bitmap doesn't overflow packet
               end if;  --no length value or length value invalid or too many blocks
               if BytePtr + (dns_types.Message_Bytes_Range(CurrentLength)+1) <=
               dns_types.Message_Bytes_Range'last then
                  BytePtr := BytePtr + (dns_types.Message_Bytes_Range(CurrentLength)+1);
               else
                  ValidMap:= False;
               end if;
            END LOOP;  --loop building bitmap

            ReturnedRec.NumberOfBlocks := CurrentBlock;
            Now := Ada.Real_Time.Clock;
            ReturnedRec.TimeStamp := Now;
         END IF; --valid nextDomainName
         if not ValidMap or ReturnedRec.NumberOfBlocks = 0 then
            Success := False;
         end if;
      end Build_NSEC_Record;

