SEPARATE(Wire_To_Records_Pkg)
   PROCEDURE Build_SOA_Record(Packet : IN Dns_Types.DNS_Message;
         RDATAStartByte : IN Dns_Types.message_bytes_range;
         TTL : IN Unsigned_Types.Unsigned32;
         Class : IN Dns_Types.Query_Class;
         ReturnedRec : OUT Rr_Type.SOA_Record_Type.SOARecordType;
         Success : OUT Boolean)
   IS
      BytePtr : Dns_Types.Message_Bytes_Range;
      CurrentByte : Dns_types.Byte;
      Now : Ada.Real_Time.Time;
   BEGIN
      ReturnedRec := rr_type.soa_record_type.BlankSOARecord;
      ReturnedRec.TTLInSeconds := TTL;
      CASE Class IS
         when dns_types.IN_CLASS =>
            ReturnedRec.Class := Rr_Type.INTERNET;
         --sun will go nova before these get executed
      	 WHEN Dns_Types.CH_CLASS =>
            ReturnedRec.Class := Rr_Type.CH;
         WHEN Dns_Types.HS_CLASS =>
            ReturnedRec.Class := Rr_Type.HS;
         --SPARK ensures this never happens due to precondition
         WHEN Dns_Types.NONE_CLASS | Dns_Types.ANY_CLASS =>
            ReturnedRec.Class := Rr_Type.INTERNET;
         END CASE;
         --copy the name server from the wire to the record, expanding any pointers
         MakePtrFreeDomainNameFromWire(Packet, RDATAStartByte, ReturnedRec.NameServer, Success);
         IF Success THEN
            --the mailbox string starts after the first PTR field (2 bytes) or the first
            --zero byte (1 byte)
            BytePtr := RDATAStartByte;
            CurrentByte := Packet.Bytes(BytePtr);
            WHILE BytePtr < Dns_Types.Message_Bytes_Range'Last LOOP
               --#assert BytePtr >= 1 and BytePtr < Dns_Types.Message_Bytes_Range'Last;
               CurrentByte := Packet.Bytes(BytePtr);
               exit when CurrentByte = 0 or IsPtrByte(CurrentByte);
         	    BytePtr := BytePtr+1;
            END LOOP;
            IF BytePtr = Dns_Types.Message_Bytes_Range'Last THEN
               Success := False;
            ELSIF CurrentByte = 0 then
               BytePtr := BytePtr + 1;
               IF BytePtr = Dns_Types.Message_Bytes_Range'Last THEN
                  Success := False;
               ELSE
                  MakePtrFreeDomainNameFromWire(Packet, BytePtr, ReturnedRec.Email, Success);
               END IF;
            ELSE --must be at a ptr field
               IF BytePtr >= Dns_Types.Message_Bytes_Range'Last-1 THEN
                  Success := False;
               ELSE
                  BytePtr := BytePtr + 2;
                  IF BytePtr = Dns_Types.Message_Bytes_Range'Last THEN
                     Success := False;
                  ELSE
                     MakePtrFreeDomainNameFromWire(Packet, BytePtr, ReturnedRec.Email, Success);
                  END IF;
               END IF;
            END IF;

            IF Success THEN
               --the rest of the data starts after the next PTR field (2 bytes) or the next
               --zero byte (1 byte)
               WHILE BytePtr < Dns_Types.Message_Bytes_Range'Last LOOP
                  --#assert BytePtr >= 1 and BytePtr < Dns_Types.Message_Bytes_Range'Last;
                  CurrentByte := Packet.Bytes(BytePtr);
                  exit when CurrentByte = 0 or IsPtrByte(CurrentByte);
            	    BytePtr := BytePtr+1;
               END LOOP;

               IF BytePtr = Dns_Types.Message_Bytes_Range'Last THEN
                  Success := False;
               ELSIF CurrentByte = 0 then
                  BytePtr := BytePtr + 1;
               ELSE --must be a PTR field
                  IF BytePtr >= Dns_Types.Message_Bytes_Range'Last-1 THEN
                     Success := False;
                  ELSE
                     BytePtr := BytePtr + 2;
                  END IF;
               END IF;

               --now BytePtr points to the first byte of the fixed length portion
               --of the SOA record, 20 bytes total
               IF BytePtr + 20 > Dns_Types.Message_Bytes_Range'Last THEN
                  Success := False;
               ELSE
                  ReturnedRec.SerialNumber := Get_Unsigned_32(Packet.Bytes,BytePtr);
                  BytePtr := BytePtr + 4;
                  ReturnedRec.Refresh:= Get_Unsigned_32(Packet.Bytes,BytePtr);
                  BytePtr := BytePtr + 4;
                  ReturnedRec.Retry:= Get_Unsigned_32(Packet.Bytes,BytePtr);
                  BytePtr := BytePtr + 4;
                  ReturnedRec.Expiry:= Get_Unsigned_32(Packet.Bytes,BytePtr);
                  BytePtr := BytePtr + 4;
                  ReturnedRec.Minimum:= Get_Unsigned_32(Packet.Bytes,BytePtr);
               END IF;
            END IF; --construction of mailbox field successful
         END IF; --construction of nameserver field successful
         Now := Ada.Real_Time.Clock;
         ReturnedRec.TimeStamp := Now;
      end Build_SOA_Record;

